;********************************************************************
; ALIENS MODULE                                                     *
;********************************************************************
;********************************************************************
; Subroutine:   AliensInit
;********************************************************************
AliensInit
                jsr AliensInitPart1
;resets the arrays for all aliens
                ldx #$00
.aliensinit_loop
                lda aliensInitialXArray,X
                sta fleetXposArray,X
                sta fleetOldXposArray,X
                sta shadowXposArray,X
                lda aliensInitialYArray,X
                sta fleetYposArray,X
                sta fleetOldYposArray,X
                sta shadowYposArray,X
                lda fleetInitialArray,X
                sta fleetArray,X
                lda #CONVOY_MODE
                sta alienModeArray,X
                lda #$00
                sta alienShapeTypeArray,X
                sta alienCollisionArray,X
                sta alienExplosionArray,X
                ;loop for each alien
                inx
                cpx #ALIENS_MAX
                bne .aliensinit_loop
                rts

;********************************************************************
; Subroutine:   AliensInitPart1
;********************************************************************
AliensInitPart1
                lda #FALSE
                sta convoyTriggerUpdatePosition
                sta convoyLeftMoveAllowed
                lda #RIGHT
                sta convoyMoveDirection
                lda #TRUE
                sta convoyRightMoveAllowed
                lda #$00
                sta CONVOYLEFTMAX
                lda #$05
                sta CONVOYRIGHTMAX
                lda #$03
                sta CONVOYPOSITION
                lda #$00
                sta CONVOYFRAMEINDEX
                sta ALIENSTEMP
                sta FLEET
                sta FLEETINDEX
                sta FLEETX
                sta FLEETY
                sta FLEETOLDX
                sta FLEETOLDY
                lda #NR_OF_PICKUP_TASKS
                sta pickupCounter
                rts

;********************************************************************
; Subroutine:   AliensIngameInit
;********************************************************************
AliensIngameInit
                ;fill it up from top with the leftover ones
                jsr RecoverAliensFleet
                ;resets the arrays for the existing ones
                ldx #$00
.aliensingameinit_loop
                lda aliensInitialXArray,X
                sta fleetXposArray,X
                sta fleetOldXposArray,X
                sta shadowXposArray,X
                lda aliensInitialYArray,X
                sta fleetYposArray,X
                sta fleetOldYposArray,X
                sta shadowYposArray,X
                lda #CONVOY_MODE
                sta alienModeArray,X
                lda #$00
                sta alienShapeTypeArray,X
                sta alienCollisionArray,X
                sta alienExplosionArray,X
                ;loop for each alien
                inx
                cpx #ALIENS_MAX
                bne .aliensingameinit_loop
                rts

;********************************************************************
; Subroutine:   RecoverAliensFleet
;********************************************************************
RecoverAliensFleet
;count the leftover aliens
                lda #$00
                ldx #$00
.recoveraliensfleet_countloop
                clc
                adc fleetArray,X
                inx
                cpx #ALIENS_MAX
                bne .recoveraliensfleet_countloop
                sta ALIENSTEMP
                cmp #$00
                bne .recoveraliensfleet_continue
                rts
.recoveraliensfleet_continue
;delete the fleet
                lda #DELETED
                ldx #$00
.recoveraliensfleet_delloop
                sta fleetArray,X
                inx
                cpx #ALIENS_MAX
                bne .recoveraliensfleet_delloop
;fill it up from top with the leftover ones
                lda #PRESENT
                ldx #$00
.recoveraliensfleet_fillloop
                sta fleetArray,X
                inx
                cpx ALIENSTEMP
                bne .recoveraliensfleet_fillloop
                rts

;********************************************************************
; Subroutine:   AliensUpdate
;********************************************************************
AliensUpdate
                lda PLAYERACTIVE
                cmp #TRUE
                beq .aliensupdate_continue
                rts
.aliensupdate_continue
                ;only for wave 1 and wave 3
                lda CURRENTWAVE
                cmp #FLOW_WAVE_2
                bne .aliensupdate_continue2
                rts
.aliensupdate_continue2
                jsr BehaviorUpdate
                jsr ConvoyUpdateFrame
                jsr ConvoyUpdatePosition
                jsr DrawActualConvoy
                jsr WaitForIrq
                lda #$00
                sta FLEETINDEX
.aliensupdate_loop
                ldx FLEETINDEX
                ;aliens get variables
                lda fleetArray,X
                sta FLEET
                cmp #PRESENT
                bne .aliensupdate_skipthisalien
                lda fleetXposArray,X
                sta FLEETX
                lda fleetYposArray,X
                sta FLEETY
                lda fleetOldXposArray,X
                sta FLEETOLDX
                lda fleetOldYposArray,X
                sta FLEETOLDY
                stx ALIENSTEMP  ;save X register as it gets trashed
                jsr AlienBulletCollisions
                jsr AlienShieldPlayerCollisions
                ;aliens set variables
                ldx ALIENSTEMP  ;restore X register as it gets trashed
                lda FLEET
                sta fleetArray,X
                lda FLEETX
                sta fleetXposArray,X
                lda FLEETY
                sta fleetYposArray,X
.aliensupdate_skipthisalien
                ;loop for each alien
                inc FLEETINDEX
                lda FLEETINDEX
                cmp #ALIENS_MAX
                bne .aliensupdate_loop
.aliensupdate_exit
                rts

;********************************************************************
; Subroutine:   DrawActualConvoy
;********************************************************************
DrawActualConvoy
                lda #$00
                sta FLEETINDEX
.drawactualconvoy_next
                ldx FLEETINDEX
                lda fleetArray,X
                cmp #PRESENT                        ;only if present
                bne .drawactualconvoy_skip
                lda alienModeArray,X
                cmp #CONVOY_MODE                    ;only if part of convoy
                beq .drawactualconvoy_ok
                lda alienModeArray,X
                cmp #GRAPPER_MODE                   ;or grapper
                bne .drawactualconvoy_skip
.drawactualconvoy_ok
                ;old becomes the new positions
                lda fleetXposArray,X
                sta fleetOldXposArray,X
                sta CURRENTSHAPEXPOS
                lda fleetYposArray,X
                sta fleetOldYposArray,X
                sta CURRENTSHAPEYPOS
                ldy CONVOYFRAMEINDEX
                lda convoyMoveDirection
                cmp #RIGHT
                bne .drawactualconvoy_left
                lda convoyMoveRightFrameLow,Y
                sta STRINGL
                lda convoyMoveRightFrameHigh,Y
                sta STRINGH
                lda convoyMoveRightColorFrameLow,Y
                sta TEXTCOLORL
                lda convoyMoveRightColorFrameHigh,Y
                sta TEXTCOLORH
                jmp .drawactualconvoy_draw
.drawactualconvoy_left
                lda convoyMoveLeftFrameLow,Y
                sta STRINGL
                lda convoyMoveLeftFrameHigh,Y
                sta STRINGH
                lda convoyMoveLeftColorFrameLow,Y
                sta TEXTCOLORL
                lda convoyMoveLeftColorFrameHigh,Y
                sta TEXTCOLORH
.drawactualconvoy_draw
                lda #$01
                sta BLOCKWIDTH
                lda #$02 
                sta BLOCKHEIGHT
                lda FLEETINDEX
                sta CURRENTALIENNUMBER
                lda #SCREEN_WIDTH-2
                sta MAXSCREENX
                jsr DrawShapeCollisionCheck
.drawactualconvoy_skip
                inc FLEETINDEX
                lda FLEETINDEX
                cmp #ALIENS_MAX
                bne .drawactualconvoy_next
                rts

;********************************************************************
; Subroutine:   ConvoyUpdateFrame
;********************************************************************
ConvoyUpdateFrame
                ldy #TIMER_6
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .convoyupdateframe_notask
                jsr RestartTimer
                ;do task ********************************************
                ;rotate aliens frame
                inc CONVOYFRAMEINDEX
                lda CONVOYFRAMEINDEX
                cmp #$04
                bne .convoyupdateframe_notask
                lda #$00
                sta CONVOYFRAMEINDEX
                lda #TRUE
                sta convoyTriggerUpdatePosition
.convoyupdateframe_notask
                rts

;********************************************************************
; Subroutine:   ConvoyUpdatePosition
; Input:        convoyTriggerUpdatePosition:    TRUE / FALSE
;               convoyMoveDirection:            LEFT / RIGHT
; Output:       convoyTriggerUpdatePosition:    TRUE / FALSE
;********************************************************************
ConvoyUpdatePosition
                lda convoyTriggerUpdatePosition
                cmp #TRUE
                bne .convoyupdateposition_exit
                lda #FALSE
                sta convoyTriggerUpdatePosition
                jsr DelOldConvoy
                lda convoyMoveDirection
                cmp #LEFT
                bne .convoyupdateposition_right
                jsr ConvoyLeftBorderCheck
                jsr ConvoyLeftPositionUpdate
                rts
.convoyupdateposition_right
                jsr ConvoyRightBorderCheck
                jsr ConvoyRightPositionUpdate
.convoyupdateposition_exit
                rts

;********************************************************************
; Subroutine:   DelOldConvoy
;********************************************************************
DelOldConvoy
                ldx #ALIENS_MAX-1
.deloldconvoy_next
                lda fleetArray,X
                cmp #PRESENT
                bne .deloldconvoy_skip
                lda alienModeArray,X
                cmp #CONVOY_MODE                    ;only if part of convoy
                beq .deloldconvoy_do
                cmp #GRAPPER_MODE                   ;or grapper
                bne .deloldconvoy_skip
.deloldconvoy_do
                lda fleetOldXposArray,X
                cmp #SCREEN_WIDTH-2
                bcs .deloldconvoy_skip
                ldy fleetOldYposArray,X
                cpy #SCREEN_HIGHT
                bcs .deloldconvoy_skipline
                lda screenRAMRowLow,Y
                sta SCREENRAML
                lda screenRAMRowHigh,Y
                sta SCREENRAMH
                ldy fleetOldXposArray,X
                lda #CHR_SPACE
                sta (SCREENRAML),Y
                lda SCREENRAMH
                eor #>(SCREENRAM XOR COLORRAM)
                sta SCREENRAMH
                lda #WHT
                sta (SCREENRAML),Y
                ldy fleetOldYposArray,X
.deloldconvoy_skipline
                iny
                cpy #SCREEN_HIGHT
                bcs .deloldconvoy_skip
                lda screenRAMRowLow,Y
                sta SCREENRAML
                lda screenRAMRowHigh,Y
                sta SCREENRAMH
                ldy fleetOldXposArray,X
                lda #CHR_SPACE
                sta (SCREENRAML),Y
                lda SCREENRAMH
                eor #>(SCREENRAM XOR COLORRAM)
                sta SCREENRAMH
                lda #WHT
                sta (SCREENRAML),Y
.deloldconvoy_skip
                dex
                bpl .deloldconvoy_next
                rts

;********************************************************************
; Subroutine:   DelNewConvoy
;********************************************************************
DelNewConvoy
                lda #$00
                sta FLEETINDEX
.delnewconvoy_next
                ldx FLEETINDEX
                lda fleetArray,X
                cmp #PRESENT
                bne .delnewconvoy_skip
                lda alienModeArray,X
                cmp #CONVOY_MODE                    ;only if part of convoy
                beq .delnewconvoy_do
                lda alienModeArray,X
                cmp #GRAPPER_MODE                   ;or grapper
                bne .delnewconvoy_skip
.delnewconvoy_do
                lda fleetXposArray,X
                sta CURRENTSHAPEXPOS
                lda fleetYposArray,X
                sta CURRENTSHAPEYPOS
                ;delete alien from screen
                lda #<clrAlien
                sta STRINGL
                lda #>clrAlien
                sta STRINGH
                lda #<clrAlienColor
                sta TEXTCOLORL
                lda #>clrAlienColor
                sta TEXTCOLORH
                lda #$01
                sta BLOCKWIDTH
                lda #$02 
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH-2
                sta MAXSCREENX
                jsr DrawShape
.delnewconvoy_skip
                inc FLEETINDEX
                lda FLEETINDEX
                cmp #ALIENS_MAX
                bne .delnewconvoy_next
                rts

;********************************************************************
; Subroutine:   ConvoyRightBorderCheck
;********************************************************************
ConvoyRightBorderCheck
                lda CONVOYPOSITION
                cmp CONVOYRIGHTMAX          ;right border reached?
                beq .convoyrightbordercheck_notallowed
                lda #TRUE
                sta convoyRightMoveAllowed
                rts
.convoyrightbordercheck_notallowed
                lda #LEFT
                sta convoyMoveDirection
                lda #FALSE
                sta convoyRightMoveAllowed
                lda #$00
                sta CONVOYFRAMEINDEX
                rts

;********************************************************************
; Subroutine:   ConvoyLeftBorderCheck
;********************************************************************
ConvoyLeftBorderCheck
                lda CONVOYPOSITION
                cmp CONVOYLEFTMAX           ;left border reached?
                beq .convoyleftbordercheck_notallowed
                lda #TRUE
                sta convoyLeftMoveAllowed
                rts
.convoyleftbordercheck_notallowed
                lda #RIGHT
                sta convoyMoveDirection
                lda #FALSE
                sta convoyLeftMoveAllowed
                lda #$00
                sta CONVOYFRAMEINDEX
                rts

;********************************************************************
; Subroutine:   ConvoyRightPositionUpdate
; Input:        convoyRightMoveAllowed:     TRUE / FALSE
;               convoyMoveDirection:        LEFT / RIGHT
;               fleetYposArray:             0 - 24
; Output:       fleetYposArray:             0 - 24
;               convoyMoveDirection:        RIGHT / LEFT
;********************************************************************
ConvoyRightPositionUpdate
                lda convoyRightMoveAllowed
                cmp #TRUE
                bne .convoyrightpositionupdate_exit
                ldx #$00
.convoyrightpositionupdate_next
                inc shadowYposArray,X       ;hold the picked up aliens
                lda alienModeArray,X
                cmp #CONVOY_MODE                    ;only if part of convoy
                beq .convoyrightpositionupdate_do
                lda alienModeArray,X
                cmp #GRAPPER_MODE                   ;or grapper
                bne .convoyrightpositionupdate_skip
                ;handle grapp time
                lda alienGrappTimeArray,X
                cmp #$00
                beq .convoyrightpositionupdate_do
                dec alienGrappTimeArray,X
.convoyrightpositionupdate_do
                inc fleetYposArray,X
.convoyrightpositionupdate_skip
                inx
                cpx #ALIENS_MAX
                bne .convoyrightpositionupdate_next
                inc CONVOYPOSITION
.convoyrightpositionupdate_exit
                rts

;********************************************************************
; Subroutine:   ConvoyLeftPositionUpdate
; Input:        convoyLeftMoveAllowed:      TRUE / FALSE
;               convoyMoveDirection:        LEFT / RIGHT
;               fleetYposArray:             0 - 24
; Output:       fleetYposArray:             0 - 24
;********************************************************************
ConvoyLeftPositionUpdate
                lda convoyLeftMoveAllowed
                cmp #TRUE
                bne .convoyleftpositionupdate_exit
                ldx #$00
.convoyleftpositionupdate_next
                dec shadowYposArray,X       ;hold the picked up aliens
                lda alienModeArray,X
                cmp #CONVOY_MODE                    ;only if part of convoy
                beq .convoyleftpositionupdate_do
                lda alienModeArray,X
                cmp #GRAPPER_MODE                   ;or grapper
                bne .convoyleftpositionupdate_skip
                ;handle grapp time
                lda alienGrappTimeArray,X
                cmp #$00
                beq .convoyleftpositionupdate_do
                dec alienGrappTimeArray,X
.convoyleftpositionupdate_do
                dec fleetYposArray,X
.convoyleftpositionupdate_skip
                inx
                cpx #ALIENS_MAX
                bne .convoyleftpositionupdate_next
                dec CONVOYPOSITION
.convoyleftpositionupdate_exit
                rts

;********************************************************************
; Subroutine:   DrawAliensFadeIn
;********************************************************************
DrawAliensFadeIn
                ldy #TIMER_B
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .drawaliensfadein_notask
                jsr RestartTimer
                ;do task ********************************************
                lda #$00
                sta FLEETINDEX
.drawaliensfadein_next
                ldx FLEETINDEX
                lda fleetArray,X
                cmp #PRESENT
                bne .drawaliensfadein_skip
                lda alienModeArray,X
                cmp #CONVOY_MODE            ;get the convoy ones
                bne .drawaliensfadein_skip
                lda fleetXposArray,X
                sta CURRENTSHAPEXPOS
                lda fleetYposArray,X
                sta CURRENTSHAPEYPOS
                ldy FADEINALIENFRAMEINDEX
                lda fadeinAlienFrameLow,Y
                sta STRINGL
                lda fadeinAlienFrameHigh,Y
                sta STRINGH
                lda fadeinAlienColorFrameLow,Y
                sta TEXTCOLORL
                lda fadeinAlienColorFrameHigh,Y
                sta TEXTCOLORH
                lda #$01
                sta BLOCKWIDTH
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH-2
                sta MAXSCREENX
                jsr DrawShape
.drawaliensfadein_skip
                inc FLEETINDEX
                lda FLEETINDEX
                cmp #ALIENS_MAX
                bne .drawaliensfadein_next
                ;next frame
                inc FADEINALIENFRAMEINDEX
                lda FADEINALIENFRAMEINDEX
                cmp #$07
                bne .drawaliensfadein_notask
                ;fade in animation is finished
                lda #$00
                sta FADEINALIENFRAMEINDEX
                lda #TRUE
                sta FADEINALIENFINISHED
.drawaliensfadein_notask
                rts

;********************************************************************
; Subroutine:   AlienBulletCollisions
;********************************************************************
AlienBulletCollisions
                ;check for collided bullets
                lda FLEETX
                sta BULLETSXCHARCOL
                lda FLEETY
                sta BULLETSYCHARCOL
                ;target is the right part of alien character
                inc BULLETSYCHARCOL
                lda #TRUE
                sta BULLETSDIRCOL
                jsr BulletsCollided
                bne .alienbulletcollisions_collision
                rts
.alienbulletcollisions_collision
                ;delete alien from screen
                lda FLEETX
                sta CURRENTSHAPEXPOS
                lda FLEETY
                sta CURRENTSHAPEYPOS
                lda #<clrAlien
                sta STRINGL
                lda #>clrAlien
                sta STRINGH
                lda #<clrAlienColor
                sta TEXTCOLORL
                lda #>clrAlienColor
                sta TEXTCOLORH
                lda #$01
                sta BLOCKWIDTH
                lda #$02 
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH-2
                sta MAXSCREENX
                jsr DrawShape
                ;remove alien from fleet
                lda #FALSE
                sta FLEET
                ;trigger alien explosion sound
                jsr TriggerSoundAlienExplosion
                ;trigger explosion animation
                ldx FLEETINDEX
                lda #EXPLOSION_TYPE_A
                sta alienExplosionArray,X
                lda CURRENTSHAPEXPOS
                sta alienExplosionXArray,X
                lda CURRENTSHAPEYPOS
                sta alienExplosionYArray,X
                ;get alien shape type
                lda alienShapeTypeArray,X
                cmp #ALIEN_SHAPE_32
                bcs .alienbulletcollisions_typeb    ;branch if birds
                lda alienModeArray,X
                cmp #ATTACK_MODE
                bne .alienbulletcollisions_next
                ;add 80 points to score
                lda #EIGHTY_POINTS
                sta VALUETOSCORE
                jsr IncreaseScoreLow
                rts
.alienbulletcollisions_next
                lda alienModeArray,X
                cmp #GRAPPER_MODE
                bne .alienbulletcollisions_next2
                ;add 20 points to score
                lda #TWENTY_POINTS
                sta VALUETOSCORE
                jsr IncreaseScoreLow
                rts
.alienbulletcollisions_next2
                ;add 50 points to score
                lda #FIFTY_POINTS
                sta VALUETOSCORE
                jsr IncreaseScoreLow
                rts
.alienbulletcollisions_typeb
                ldx FLEETINDEX
                lda #$00
                sta alienShapeTypeArray,X
                lda #EXPLOSION_TYPE_B
                sta alienExplosionArray,X
                ;add 200 points to score
                lda #TWOHUNDRED_POINTS
                sta VALUETOSCORE
                jsr IncreaseScoreHigh
                ;trigger score sound
                jsr TriggerSoundScore
                rts

;********************************************************************
; Subroutine:   AlienShieldPlayerCollisions
;********************************************************************
AlienShieldPlayerCollisions
                ldx FLEETINDEX
                lda alienCollisionArray,X
                cmp #TRUE
                bne .alienshieldplayercollisions_exit
                ;alien gets destroyed
                lda #FALSE
                sta alienCollisionArray,X
                ;trigger explosion animation
                lda #EXPLOSION_TYPE_A
                sta alienExplosionArray,X
                ;delete alien from screen
                lda fleetXposArray,X
                sta CURRENTSHAPEXPOS
                lda fleetYposArray,X
                sta CURRENTSHAPEYPOS
                lda #<clrAlien
                sta STRINGL
                lda #>clrAlien
                sta STRINGH
                lda #<clrAlienColor
                sta TEXTCOLORL
                lda #>clrAlienColor
                sta TEXTCOLORH
                lda #$01
                sta BLOCKWIDTH
                lda #$02 
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH-2
                sta MAXSCREENX
                jsr DrawShape
                ;remove alien from fleet
                lda #FALSE
                sta FLEET
                ;add 40 points to score
                lda #FOURTY_POINTS
                sta VALUETOSCORE
                jsr IncreaseScoreLow
                ;trigger alien explosion sound
                jsr TriggerSoundAlienExplosion
.alienshieldplayercollisions_exit
                rts

;********************************************************************
;alien data *********************************************************
;********************************************************************
fleetInitialArray
                !fill ALIENS_MAX, PRESENT       ;alien #1,#2,#3,..#16

aliensInitialXArray
                !fill ALIENS_MAX,0              ;0-39

aliensInitialYArray
                !fill ALIENS_MAX,0              ;0-24

;alien formation A
alienFormAArray
;x positions for alien#01   02   03   04   05   06   07   08   09   10   11   12   13   14   15   16
                !byte  27,  29,  27,  27,  28,  28,  29,  29,  30,  30,  32,  32,  33,  33,  34,  34
;y positions                                                                                        
                !byte  11,  11,   7,  15,   5,  17,   3,  19,   1,  21,   1,  21,   3,  19,   5,  17

;alien formation B
alienFormBArray
;x positions for alien#01   02   03   04   05   06   07   08   09   10   11   12   13   14   15   16
                !byte  27,  29,  30,  30,  31,  31,  31,  33,  34,  34,  32,  32,  31,  31,  29,  29
;y positions                                                                                        
                !byte  11,  11,   9,  13,  11,   7,  15,  11,   9,  13,   5,  17,   3,  19,   2,  20

;alien formation C
alienFormCArray
;x positions for alien#01   02   03   04   05   06   07   08   09   10   11   12   13   14   15   16
                !byte  27,  28,  28,  29,  29,  29,  30,  30,  31,  31,  32,  32,  33,  33,  34,  34
;y positions                                                                                        
                !byte  11,   9,  13,  11,   7,  15,   9,  13,   7,  15,   5,  17,   3,  19,   1,  21

;alien formation D
alienFormDArray
;x positions for alien#01   02   03   04   05   06   07   08   09   10   11   12   13   14   15   16
                !byte  27,  27,  27,  28,  28,  29,  29,  30,  30,  31,  31,  32,  32,  33,  33,  34
;y positions                                                                                        
                !byte  11,   9,  13,   7,  15,   5,  17,   3,  19,   5,  17,   7,  15,   9,  13,  11

;alien formation E (for wave 3)
alienFormEArray
                !byte  30,  31,  32,  33,  34,  33,  32,  31    ;x positions
                !byte  30,  31,  32,  33,  34,  33,  32,  31
                !byte   4,   6,   8,  10,  12,  14,  16,  18    ;y positions
                !byte  20,   6,   8,  10,  12,  14,  16,  18

;alien formation F (for wave 3)
alienFormFArray
                !byte  30,  31,  32,  33,  34,  33,  32,  31    ;x positions
                !byte  30,  22,  22,  22,  22,  22,  22,  22
                !byte   4,   6,   8,  10,  12,  14,  16,  18    ;y positions
                !byte  20,   6,   8,  10,  12,  14,  16,  18

;alien formation table
alienFormationTableLow
                !byte <alienFormAArray
                !byte <alienFormBArray
                !byte <alienFormCArray
                !byte <alienFormDArray
                !byte <alienFormEArray
                !byte <alienFormFArray

alienFormationTableHigh
                !byte >alienFormAArray
                !byte >alienFormBArray
                !byte >alienFormCArray
                !byte >alienFormDArray
                !byte >alienFormEArray
                !byte >alienFormFArray

fleetArray
                !fill ALIENS_MAX,0

fleetXposArray
                !fill ALIENS_MAX,0

shadowXposArray
                !fill ALIENS_MAX,0

fleetYposArray
                !fill ALIENS_MAX,0

shadowYposArray
                !fill ALIENS_MAX,0

fleetOldXposArray
                !fill ALIENS_MAX,0

fleetOldYposArray
                !fill ALIENS_MAX,0

alienModeArray
                !fill ALIENS_MAX, CONVOY_MODE

alienCollisionArray
                !fill ALIENS_MAX, 0

alienShapeTypeArray
                !fill ALIENS_MAX, 0

convoyMoveDirection
                !byte RIGHT

convoyLeftMoveAllowed
                !byte FALSE

convoyRightMoveAllowed
                !byte TRUE

convoyTriggerUpdatePosition
                !byte FALSE

;fade in alien frame table
fadeinAlienFrameLow
                !byte <alienShape09
                !byte <alienShape10
                !byte <alienShape11
                !byte <alienShape12
                !byte <alienShape05
                !byte <alienShape07

fadeinAlienFrameHigh
                !byte >alienShape09
                !byte >alienShape10
                !byte >alienShape11
                !byte >alienShape12
                !byte >alienShape05
                !byte >alienShape07
;fade in alien color table
fadeinAlienColorFrameLow
                !byte <alienShape09Color
                !byte <alienShape10Color
                !byte <alienShape11Color
                !byte <alienShape12Color
                !byte <alienShape05Color
                !byte <alienShape07Color

fadeinAlienColorFrameHigh
                !byte >alienShape09Color
                !byte >alienShape10Color
                !byte >alienShape11Color
                !byte >alienShape12Color
                !byte >alienShape05Color
                !byte >alienShape07Color

;delete convoy alien
clrAlien
                !byte CHR_SPACE,CHR_SPACE

clrAlienColor
                !byte WHT,WHT

;convoy move right frame table
convoyMoveRightFrameLow
                !byte <alienShape01
                !byte <alienShape02
                !byte <alienShape03
                !byte <alienShape04

convoyMoveRightFrameHigh
                !byte >alienShape01
                !byte >alienShape02
                !byte >alienShape03
                !byte >alienShape04

convoyMoveRightColorFrameLow
                !byte <alienShape01Color
                !byte <alienShape02Color
                !byte <alienShape03Color
                !byte <alienShape04Color

convoyMoveRightColorFrameHigh
                !byte >alienShape01Color
                !byte >alienShape02Color
                !byte >alienShape03Color
                !byte >alienShape04Color

;convoy move left frame table
convoyMoveLeftFrameLow
                !byte <alienShape13
                !byte <alienShape04
                !byte <alienShape03
                !byte <alienShape02

convoyMoveLeftFrameHigh
                !byte >alienShape13
                !byte >alienShape04
                !byte >alienShape03
                !byte >alienShape02

convoyMoveLeftColorFrameLow
                !byte <alienShape13Color
                !byte <alienShape04Color
                !byte <alienShape03Color
                !byte <alienShape02Color

convoyMoveLeftColorFrameHigh
                !byte >alienShape13Color
                !byte >alienShape04Color
                !byte >alienShape03Color
                !byte >alienShape02Color

;alien shape character table
alienShapeChrTable
;delete shape
alienShapeDel
                !byte CHR_SPACE,CHR_SPACE
                !byte CHR_SPACE,CHR_SPACE

alienShape01
                !byte $91
                !byte $92

alienShape02
                !byte $93
                !byte $94

alienShape03
                !byte $95
                !byte $96

alienShape04
                !byte $97
                !byte $98

alienShape05
                !byte $99

alienShape06
                !byte $9A

alienShape07
                !byte $9B

alienShape08
                !byte $9C

alienShape09
                !byte $9D

alienShape10
                !byte $9E

alienShape11
                !byte $9F

alienShape12
                !byte $A0

alienShape13
                !byte $A1
                !byte $A2

alienShape14
                !byte $A5,$A3
                !byte $A6,$A4

alienShape15
                !byte $A9,$A7
                !byte $AA,$A8
alienShape16
                !byte $AB
                !byte $AC

alienShape17
                !byte $AD
                !byte $AE

alienShape18
                !byte $AF
                !byte $B0

alienShape19
                !byte $B1
                !byte $B2

alienShape20
                !byte $B4,$B3

alienShape21
                !byte $B6,$B5

alienShape22
                !byte $B8,$B7

alienShape23
                !byte $BA,$B9

alienShape24
                !byte $BD,$BB
                !byte $BE,$BC

alienShape25
                !byte $C1,$BF
                !byte $C2,$C0

alienShape26
                !byte $C5,$C3
                !byte $C6,$C4

alienShape27
                !byte $C9,$C7
                !byte $CA,$C8

alienShape28
                !byte $CB
                !byte $CC

alienShape29
                !byte $CD
                !byte $CE

alienShape30
                !byte $D0,$CF

alienShape31
                !byte $D2,$D1

alienShape32
                !byte $D3

alienShape33
                !byte $D4

alienShape34
                !byte $D7,$D5
                !byte $D8,$D6

alienShape35
                !byte $DB,$D9
                !byte $DC,$DA

alienShape36
                !byte $DF,$DD
                !byte $E0,$DE

alienShape37
                !byte $E3,$E1
                !byte $E4,$E2

alienShape38
                !byte $E7,$E5
                !byte $E8,$E6

alienShape39
                !byte $EB,$E9
                !byte $EC,$EA

;alien shape address table
alienShapeAddrTableLow
                !byte <alienShapeDel
                !byte <alienShape01
                !byte <alienShape02
                !byte <alienShape03
                !byte <alienShape04
                !byte <alienShape05
                !byte <alienShape06
                !byte <alienShape07
                !byte <alienShape08
                !byte <alienShape09
                !byte <alienShape10
                !byte <alienShape11
                !byte <alienShape12
                !byte <alienShape13
                !byte <alienShape14
                !byte <alienShape15
                !byte <alienShape16
                !byte <alienShape17
                !byte <alienShape18
                !byte <alienShape19
                !byte <alienShape20
                !byte <alienShape21
                !byte <alienShape22
                !byte <alienShape23
                !byte <alienShape24
                !byte <alienShape25
                !byte <alienShape26
                !byte <alienShape27
                !byte <alienShape28
                !byte <alienShape29
                !byte <alienShape30
                !byte <alienShape31
                !byte <alienShape32
                !byte <alienShape33
                !byte <alienShape34
                !byte <alienShape35
                !byte <alienShape36
                !byte <alienShape37
                !byte <alienShape38
                !byte <alienShape39

alienShapeAddrTableHigh
                !byte >alienShapeDel
                !byte >alienShape01
                !byte >alienShape02
                !byte >alienShape03
                !byte >alienShape04
                !byte >alienShape05
                !byte >alienShape06
                !byte >alienShape07
                !byte >alienShape08
                !byte >alienShape09
                !byte >alienShape10
                !byte >alienShape11
                !byte >alienShape12
                !byte >alienShape13
                !byte >alienShape14
                !byte >alienShape15
                !byte >alienShape16
                !byte >alienShape17
                !byte >alienShape18
                !byte >alienShape19
                !byte >alienShape20
                !byte >alienShape21
                !byte >alienShape22
                !byte >alienShape23
                !byte >alienShape24
                !byte >alienShape25
                !byte >alienShape26
                !byte >alienShape27
                !byte >alienShape28
                !byte >alienShape29
                !byte >alienShape30
                !byte >alienShape31
                !byte >alienShape32
                !byte >alienShape33
                !byte >alienShape34
                !byte >alienShape35
                !byte >alienShape36
                !byte >alienShape37
                !byte >alienShape38
                !byte >alienShape39
;alien shape color table
alienShapeColorTable
;delete shape
alienShapeDelColor
                !byte WHT,WHT
                !byte WHT,WHT

alienShape01Color
                !byte 0
                !byte 0

alienShape02Color
                !byte 0
                !byte 0

alienShape03Color
                !byte 0
                !byte 0

alienShape04Color
                !byte 0
                !byte 0

alienShape05Color
                !byte 0

alienShape06Color
                !byte 0

alienShape07Color
                !byte 0

alienShape08Color
                !byte 0

alienShape09Color
                !byte 0

alienShape10Color
                !byte 0

alienShape11Color
                !byte 0

alienShape12Color
                !byte 0

alienShape13Color
                !byte 0
                !byte 0

alienShape14Color
                !byte 0,0
                !byte 0,0

alienShape15Color
                !byte 0,0
                !byte 0,0

alienShape16Color
                !byte 0
                !byte 0

alienShape17Color
                !byte 0
                !byte 0

alienShape18Color
                !byte 0
                !byte 0

alienShape19Color
                !byte 0
                !byte 0

alienShape20Color
                !byte 0,0

alienShape21Color
                !byte 0,0

alienShape22Color
                !byte 0,0

alienShape23Color
                !byte 0,0

alienShape24Color
                !byte 0,0
                !byte 0,0

alienShape25Color
                !byte 0,0
                !byte 0,0

alienShape26Color
                !byte 0,0
                !byte 0,0

alienShape27Color
                !byte 0,0
                !byte 0,0

alienShape28Color
                !byte 0
                !byte 0

alienShape29Color
                !byte 0
                !byte 0

alienShape30Color
                !byte 0,0

alienShape31Color
                !byte 0,0

alienShape32Color
                !byte 0

alienShape33Color
                !byte 0

alienShape34Color
                !byte 0,0
                !byte 0,0

alienShape35Color
                !byte 0,0
                !byte 0,0

alienShape36Color
                !byte 0,0
                !byte 0,0

alienShape37Color
                !byte 0,0
                !byte 0,0

alienShape38Color
                !byte 0,0
                !byte 0,0

alienShape39Color
                !byte 0,0
                !byte 0,0

;Alien color sets
alienShapeColorSet1
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte PUR
                !byte PUR
                !byte PUR
                !byte PUR
                !byte YEL6
                !byte YEL6
                !byte YEL6,YEL6
                !byte YEL6,YEL6
                !byte YEL6,YEL6
                !byte YEL6,YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6
                !byte YEL6,YEL6
                !byte YEL6,YEL6
                !byte YEL6,YEL6
                !byte YEL6,YEL6
                !byte PNK,YEL6
                !byte YEL6,PNK
                !byte YEL6,PNK
                !byte PNK,YEL6
                !byte PNK,YEL6
                !byte YEL6,PNK
                !byte YEL6,PNK
                !byte PNK,YEL6
                !byte PNK
                !byte YEL6
                !byte YEL6
                !byte PNK
                !byte PNK,YEL6
                !byte YEL6,PNK
                !byte PNK
                !byte PNK
                !byte PNK,YEL6
                !byte YEL6,PNK
                !byte PNK,YEL6
                !byte YEL6,YEL6
                !byte PNK,YEL6
                !byte YEL6,YEL6
                !byte YEL6,PNK
                !byte PNK,YEL6
                !byte YEL6,YEL6
                !byte PNK,YEL6
                !byte YEL6,YEL6
                !byte PNK,YEL6

alienShapeColorSet2
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte PUR
                !byte PUR
                !byte PUR
                !byte PUR
                !byte LGRN
                !byte LGRN
                !byte LGRN,LGRN
                !byte LGRN,LGRN
                !byte LGRN,LGRN
                !byte LGRN,LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN
                !byte LGRN,LGRN
                !byte LGRN,LGRN
                !byte LGRN,LGRN
                !byte LGRN,LGRN
                !byte PNK,LGRN
                !byte LGRN,PNK
                !byte LGRN,PNK
                !byte PNK,LGRN
                !byte PNK,LGRN
                !byte LGRN,PNK
                !byte LGRN,PNK
                !byte PNK,LGRN
                !byte PNK
                !byte LGRN
                !byte LGRN
                !byte PNK
                !byte PNK,LGRN
                !byte LGRN,PNK
                !byte PNK
                !byte PNK
                !byte PNK,LGRN
                !byte LGRN,PNK
                !byte PNK,LGRN
                !byte LGRN,LGRN
                !byte PNK,LGRN
                !byte LGRN,LGRN
                !byte LGRN,PNK
                !byte PNK,LGRN
                !byte LGRN,LGRN
                !byte PNK,LGRN
                !byte LGRN,LGRN
                !byte PNK,LGRN

;alien shape color address table
alienShapeColorAddrTableLow
                !byte <alienShapeDelColor
                !byte <alienShape01Color
                !byte <alienShape02Color
                !byte <alienShape03Color
                !byte <alienShape04Color
                !byte <alienShape05Color
                !byte <alienShape06Color
                !byte <alienShape07Color
                !byte <alienShape08Color
                !byte <alienShape09Color
                !byte <alienShape10Color
                !byte <alienShape11Color
                !byte <alienShape12Color
                !byte <alienShape13Color
                !byte <alienShape14Color
                !byte <alienShape15Color
                !byte <alienShape16Color
                !byte <alienShape17Color
                !byte <alienShape18Color
                !byte <alienShape19Color
                !byte <alienShape20Color
                !byte <alienShape21Color
                !byte <alienShape22Color
                !byte <alienShape23Color
                !byte <alienShape24Color
                !byte <alienShape25Color
                !byte <alienShape26Color
                !byte <alienShape27Color
                !byte <alienShape28Color
                !byte <alienShape29Color
                !byte <alienShape30Color
                !byte <alienShape31Color
                !byte <alienShape32Color
                !byte <alienShape33Color
                !byte <alienShape34Color
                !byte <alienShape35Color
                !byte <alienShape36Color
                !byte <alienShape37Color
                !byte <alienShape38Color
                !byte <alienShape39Color

alienShapeColorAddrTableHigh
                !byte >alienShapeDelColor
                !byte >alienShape01Color
                !byte >alienShape02Color
                !byte >alienShape03Color
                !byte >alienShape04Color
                !byte >alienShape05Color
                !byte >alienShape06Color
                !byte >alienShape07Color
                !byte >alienShape08Color
                !byte >alienShape09Color
                !byte >alienShape10Color
                !byte >alienShape11Color
                !byte >alienShape12Color
                !byte >alienShape13Color
                !byte >alienShape14Color
                !byte >alienShape15Color
                !byte >alienShape16Color
                !byte >alienShape17Color
                !byte >alienShape18Color
                !byte >alienShape19Color
                !byte >alienShape20Color
                !byte >alienShape21Color
                !byte >alienShape22Color
                !byte >alienShape23Color
                !byte >alienShape24Color
                !byte >alienShape25Color
                !byte >alienShape26Color
                !byte >alienShape27Color
                !byte >alienShape28Color
                !byte >alienShape29Color
                !byte >alienShape30Color
                !byte >alienShape31Color
                !byte >alienShape32Color
                !byte >alienShape33Color
                !byte >alienShape34Color
                !byte >alienShape35Color
                !byte >alienShape36Color
                !byte >alienShape37Color
                !byte >alienShape38Color
                !byte >alienShape39Color

alienSizeTable  ;BLOCKWIDTH, BLOCKHEIGHT
alienShapeDelSize
                !byte 2,2

alienShape01Size
                !byte 1,2

alienShape02Size
                !byte 1,2

alienShape03Size
                !byte 1,2

alienShape04Size
                !byte 1,2

alienShape05Size
                !byte 1,1

alienShape06Size
                !byte 1,1

alienShape07Size
                !byte 1,1

alienShape08Size
                !byte 1,1

alienShape09Size
                !byte 1,1

alienShape10Size
                !byte 1,1

alienShape11Size
                !byte 1,1

alienShape12Size
                !byte 1,1

alienShape13Size
                !byte 1,2

alienShape14Size
                !byte 2,2

alienShape15Size
                !byte 2,2

alienShape16Size
                !byte 1,2

alienShape17Size
                !byte 1,2

alienShape18Size
                !byte 1,2

alienShape19Size
                !byte 1,2

alienShape20Size
                !byte 2,1

alienShape21Size
                !byte 2,1

alienShape22Size
                !byte 2,1

alienShape23Size
                !byte 2,1

alienShape24Size
                !byte 2,2

alienShape25Size
                !byte 2,2

alienShape26Size
                !byte 2,2

alienShape27Size
                !byte 2,2

alienShape28Size
                !byte 1,2

alienShape29Size
                !byte 1,2

alienShape30Size
                !byte 2,1

alienShape31Size
                !byte 2,1

alienShape32Size
                !byte 1,1

alienShape33Size
                !byte 1,1

alienShape34Size
                !byte 2,2

alienShape35Size
                !byte 2,2

alienShape36Size
                !byte 2,2

alienShape37Size
                !byte 2,2

alienShape38Size
                !byte 2,2

alienShape39Size
                !byte 2,2

;********************************************************************
; ALIENS MODULE END                                                 *
;********************************************************************