;********************************************************************
; BULLETS MODULE                                                    *
;********************************************************************
;********************************************************************
; Subroutine:   BulletsUpdate
;               Called during raster interrupt in order to get smooth
;               and constant bullets movement.
;               Parameters: 'direction', 'position', 'speed', 'character'
;               and 'color' is handled within that queue.
;               'offset' could used for position-mapping (sprites).
;********************************************************************
BulletsUpdate
                lda PLAYERACTIVE
                bne .bulletsupdate_do
                rts
.bulletsupdate_do
                ldx #$00
.bulletsupdate_loop
                lda bulletsActive,X
                bne .bulletsupdate_ok
                jmp .bulletsupdate_skipbulletupdate
                ;get the current bullet from the list
.bulletsupdate_ok
                ;get current bullets data
                lda bulletsXChar,X
                sta BULLETSXCHARCURRENT
                lda bulletsYOffset,X
                sta BULLETSYOFFSETCURRENT
                lda bulletsYChar,X
                sta BULLETSYCHARCURRENT
                lda bulletsColor,X
                sta BULLETSCOLORCURRENT
                lda bulletsDir,X
                sta BULLETSDIRCURRENT
                ;handle bullet speed
                lda bulletsWait,X
                beq .bulletsupdate_waitdone
                dec bulletsWait,X
                jmp .bulletsupdate_skipbulletupdate
.bulletsupdate_waitdone
                ;reset wait
                lda bulletsSpeed,X
                sta bulletsWait,X
                ;set bullet screen ram pointer
                ldy BULLETSYCHARCURRENT
                lda screenRAMRowLow,Y
                sta BULLETSCREENL
                lda screenRAMRowHigh,Y
                sta BULLETSCREENH
                ldy BULLETSXCHARCURRENT
                lda #CHR_SPACE
                sta (BULLETSCREENL),Y
                lda BULLETSDIRCURRENT
                beq .bulletsupdate_down
.bulletsupdate_up
                ldy BULLETSXCHARCURRENT
                iny
                sty BULLETSXCHARCURRENT
                cpy #SCREEN_WIDTH-3
                bne .bulletsupdate_draw
                jmp .bulletsupdate_dirdone
.bulletsupdate_down
                ldy BULLETSXCHARCURRENT
                dey
                sty BULLETSXCHARCURRENT
                cpy #$00
                bne .bulletsupdate_draw
.bulletsupdate_dirdone
                lda #$00
                sta bulletsActive,X
                jmp .bulletsupdate_skipbulletupdate
.bulletsupdate_draw
                ;draw the current bullet color
                ;set bullet color ram pointer
                ldy BULLETSYCHARCURRENT
                lda colorRAMRowLow,Y
                sta BULLETSCREENL
                lda colorRAMRowHigh,Y
                sta BULLETSCREENH
                ldy BULLETSXCHARCURRENT
                lda BULLETSCOLORCURRENT
                sta (BULLETSCREENL),Y
                ;set the bullet character
                ;set bullet screen ram pointer
                ldy BULLETSYCHARCURRENT
                lda screenRAMRowLow,Y
                sta BULLETSCREENL
                lda screenRAMRowHigh,Y
                sta BULLETSCREENH
                ldy BULLETSXCHARCURRENT
                lda BULLETSYOFFSETCURRENT
                sta (BULLETSCREENL),Y
                lda BULLETSXCHARCURRENT
                sta bulletsXChar,X
.bulletsupdate_skipbulletupdate
                ;loop for each bullet
                inx
                cpx #BULLETS_MAX
                beq .bulletsupdate_finished
                jmp .bulletsupdate_loop
.bulletsupdate_finished
                rts

;********************************************************************
; Subroutine:   BulletsCollided
;********************************************************************
BulletsCollided
                ldx #$00
.bulletscollided_loop
                ;skip this bullet if not active
                lda bulletsActive,X
                beq .bulletscollided_skip
                ;skip if up/down not equal
                lda bulletsDir,X
                cmp BULLETSDIRCOL
                bne .bulletscollided_skip
                ;skip if currentbullet YChar != YChar
                lda bulletsYChar,X
                cmp BULLETSYCHARCOL
                bne .bulletscollided_skip
                lda #$00
                sta BULLETSYFLAG
                ;skip if currentbullet XChar != XChar
                ldy bulletsXChar,X
                cpy BULLETSXCHARCOL
                bne .bulletscollided_xminus1
                lda #$01
                sta BULLETSYFLAG
                jmp .bulletscollided_donexcheck
.bulletscollided_xminus1
                ;skip if currentbullet XChar-1 != XChar
                dey
                cpy BULLETSXCHARCOL
                bne .bulletscollided_xplus1
                lda #$01
                sta BULLETSYFLAG
                jmp .bulletscollided_donexcheck
.bulletscollided_xplus1
                ;skip if currentbullet XChar+1 != XChar
                iny
                iny
                cpy BULLETSXCHARCOL
                bne .bulletscollided_donexcheck
                lda #$01
                sta BULLETSYFLAG
.bulletscollided_donexcheck
                lda BULLETSYFLAG
                beq .bulletscollided_skip
                ;collided
                lda #$00
                sta bulletsActive,X ;disable bullet
                ;delete bullet from screen
                lda bulletsXChar,X
                sta BULLETSXCHARCURRENT
                lda bulletsYChar,X
                sta BULLETSYCHARCURRENT
                ;set bullet screen ram pointer
                ldy BULLETSYCHARCURRENT
                lda screenRAMRowLow,Y
                sta BULLETSCREENL
                lda screenRAMRowHigh,Y
                sta BULLETSCREENH
                ldy BULLETSXCHARCURRENT
                lda #CHR_SPACE
                sta (BULLETSCREENL),Y
                lda #$01 ;set as collided
                rts
.bulletscollided_skip
                ;loop for each bullet
                inx
                cpx #BULLETS_MAX
                bne .bulletscollided_loop
                ;set as not collided
                lda #$00
.bulletscollided_collided
                rts

;********************************************************************
; Subroutine:   BulletsReset
;********************************************************************
BulletsReset
                ldx #$00
.bulletsreset_loop
                lda bulletsActive,x
                beq .bulletsreset_skip
                ;remove the bullet from the screen
                ;get current bullets data
                lda bulletsXChar,X
                sta BULLETSXCHARCURRENT
                lda bulletsYOffset,X
                sta BULLETSYOFFSETCURRENT
                lda bulletsYChar,X
                sta BULLETSYCHARCURRENT
                lda bulletsColor,X
                sta BULLETSCOLORCURRENT
                lda bulletsDir,X
                sta BULLETSDIRCURRENT
                ;set bullet screen ram pointer
                ldy BULLETSYCHARCURRENT
                lda screenRAMRowLow,Y
                sta BULLETSCREENL
                lda screenRAMRowHigh,Y
                sta BULLETSCREENH
                ldy BULLETSXCHARCURRENT
                lda #CHR_SPACE
                sta (BULLETSCREENL),Y
                lda #$00
                sta bulletsActive,x
.bulletsreset_skip
                inx
                cpx #BULLETS_MAX
                bne .bulletsreset_loop       ;loop for each bullet
                rts

;********************************************************************
; Subroutine:   PlayerBulletsFire
;********************************************************************
PlayerBulletsFire
                ldx #$00
.playerbulletsfire_loop
                lda bulletsActive,X
                bne .playerbulletsfire_skip
                ;save the current bullet in the list
                lda #$01
                sta bulletsActive,X
                lda PLAYERXCHAR
                sta bulletsXChar,X
                lda #CHR_PLAYERBULLET
                sta bulletsYOffset,X
                lda PLAYERYCHAR
                sta bulletsYChar,X
                ;player bullet color
                lda #WHT
                sta bulletsColor,X
                ;player bullet direction
                lda #TRUE
                sta bulletsDir,X
                ;player bullet speed
                lda #$00
                sta bulletsSpeed,X
                sta bulletsWait,X
                ;found a slot, quit the loop
                rts
.playerbulletsfire_skip
                ;loop for each bullet
                inx
                cpx #BULLETS_MAX
                bne .playerbulletsfire_loop
.playerbulletsfire_found
                rts

;********************************************************************
; Subroutine:   AlienBulletsFire
;********************************************************************
AlienBulletsFire
                ;X position >= 0 ?
                lda CURRENTSHAPEXPOS
                cmp #$00
                bcc .alienbulletsfire_exit     ;branch if X < 0
                ;X position < 39 ?
                lda CURRENTSHAPEXPOS
                cmp #SCREEN_WIDTH-1
                bcs .alienbulletsfire_exit     ;branch if X >= 39
                ;Y position >= 0 ?
                lda CURRENTSHAPEYPOS
                cmp #$00
                bcc .alienbulletsfire_exit     ;branch if Y < 0
                ;Y position < 24 ?
                lda CURRENTSHAPEYPOS
                cmp #SCREEN_HIGHT-1
                bcs .alienbulletsfire_exit     ;branch if Y >= 24
                ;fire only if player ship underneath alien
                lda playerY
                cmp CURRENTSHAPEYPOS
                bne .alienbulletsfire_exit
                ldx #$00
.alienbulletsfire_loop
                lda bulletsActive,X
                bne .alienbulletsfire_skip
                ;save the current bullet in the list
                lda #$01
                sta bulletsActive,X
                lda CURRENTSHAPEXPOS
                sta bulletsXChar,X
                lda #CHR_ALIENBULLET1
                sta bulletsYOffset,X
                lda CURRENTSHAPEYPOS
                sta bulletsYChar,X
                ;alien bullet color
                lda #YEL6
                sta bulletsColor,X
                ;alien bullet direction
                lda #FALSE
                sta bulletsDir,X
                ;alien bullet speed
                lda #$01
                sta bulletsSpeed,X
                sta bulletsWait,X
                ;trigger the alien firing sound
                jsr TriggerSoundAlienFiring
                ;found a slot, quit the loop
                rts
.alienbulletsfire_skip
                ;loop for each bullet
                inx
                cpx #BULLETS_MAX
                bne .alienbulletsfire_loop
.alienbulletsfire_exit
                rts

;********************************************************************
;bullets data *******************************************************
;********************************************************************
bulletsActive
                !fill BULLETS_MAX, 0

bulletsXChar
                !fill BULLETS_MAX, 0

bulletsYChar
                !fill BULLETS_MAX, 0

bulletsYOffset
                !fill BULLETS_MAX, 0

bulletsColor
                !fill BULLETS_MAX, 0

bulletsDir
                !fill BULLETS_MAX, 0

bulletsSpeed
                !fill BULLETS_MAX, 0

bulletsWait
                !fill BULLETS_MAX, 0

;********************************************************************
; BULLETS MODULE END                                                *
;********************************************************************