;********************************************************************
; FLOW MODULE                                                       *
;********************************************************************
;********************************************************************
; Jump table handler:   GameFlowUpdate
;********************************************************************
GameFlowUpdate
                ;get the current game flow state
                ldy FLOWSTATE
                ;write the subroutine address to game flow pointer
                lda gameFlowJumpTableLow,y
                sta GAMEFLOWL
                lda gameFlowJumpTableHigh,y
                sta GAMEFLOWH
                ;jump to the subroutine at game flow pointer
                jmp (GAMEFLOWL)

;********************************************************************
; Subroutine:   TreatFlowStateTitle
;********************************************************************
TreatFlowStateTitle
                ;****************************
                ;* FLOWSTATE:               *
                ;*  FLOW_STATE_TITLE        *
                ;*                          *
                ;****************************
                lda FLOWTITLESTARTED
                cmp #FALSE
                bne .treatflowstatetitle_checkend
                lda #TRUE
                sta FLOWTITLESTARTED
                ;init all software timers
                jsr ResetAllTimers
                ;set charactes to set A
                lda #CHAR_SETA_BASE
                sta CHRSETBASE
                jsr SetCharSet
                ;start title timer
                ldy #TIMER_0
                jsr StartTimer
                ;load title color set 1: LGRN, PINK or
                ;                 set 2: YEL, PURPLE
                jsr LoadTitleColorSet
                ;reset execution table pointer
                jsr DoCommandResetPointers
.treatflowstatetitle_checkend
                ldy #TIMER_0
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .treatflowstatetitle_waitforfire
                jsr ResetTimer
                ;do task ********************************************
                ;get timer value
                jsr GetIntroExecutionTimer
                ;set timer value and restart
                ldy #TIMER_0
                lda #$00
                ldx TITLETIMER
                jsr SetTimerValue
                jsr StartTimer
                ;get CMD
                jsr GetIntroExecutionCommand
                ;execute CMD
                jsr IntroCommandCase
.treatflowstatetitle_waitforfire
                ;wait for fire pressed
                jsr GetFirePressed
                bne .treatflowstatetitle_end
                inc COIN
                jsr ClearScreen
                jsr DoCommandPrintTopLines
                ;trigger sound: 'Get ready'
                jsr TriggerSoundGetReady
                ;change state to: PUSHBUTTON
                lda #FLOW_STATE_PUSHBUTTON
                sta FLOWSTATE
.treatflowstatetitle_end
                rts

;********************************************************************
; Subroutine:   TreatFlowStateSlowPrint
;********************************************************************
TreatFlowStateSlowPrint
                ;****************************
                ;* FLOWSTATE:               *
                ;*  FLOW_STATE_SLOWPRINT    *
                ;*                          *
                ;****************************
                ldy #TIMER_0
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .treatflowstateslowprint_exit
                jsr RestartTimer
                ;do task ********************************************
                ;change state to PUSHBUTTON if fire pressed
                jsr GetFirePressed
                bne .treatflowstateslowprint_continue
                inc COIN
                jsr ClearScreen
                jsr DoCommandPrintTopLines
                ;trigger sound: 'Get ready'
                jsr TriggerSoundGetReady
                ;reset column index
                lda #$00
                sta TITLECOLUMNINDEX
                lda #FLOW_STATE_PUSHBUTTON
                sta FLOWSTATE
                rts
.treatflowstateslowprint_continue
                ;print one character
                jsr PrintCharacter
                ;increment column index
                inc TITLECOLUMNINDEX
                ;if column index >25
                lda TITLECOLUMNINDEX
                cmp #$19
                bne .treatflowstateslowprint_notask
                ;column index = 0
                lda #$00
                sta TITLECOLUMNINDEX
                ;increment line index
                jsr IncTitleLinePointer
                ;check if end of line reached for page 1,2
                lda TITLECHRLINEINDEX
                cmp #$08
                bne .treatflowstateslowprint_notask
                ;change state to: TITLE
                lda #FLOW_STATE_TITLE
                sta FLOWSTATE
                rts
.treatflowstateslowprint_notask
                lda TITLECHRLINEINDEX
                cmp #$12
                bne .treatflowstateslowprint_exit
                ;change state to: TITLE
                lda #FLOW_STATE_TITLE
                sta FLOWSTATE
.treatflowstateslowprint_exit
                rts

;********************************************************************
; Subroutine:   TreatFlowStateAnimation
;********************************************************************
TreatFlowStateAnimation
                ;****************************
                ;* FLOWSTATE:               *
                ;*  FLOW_STATE_ANIMATION    *
                ;*                          *
                ;****************************
                ldy #TIMER_0
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .treatflowstateanimation_exit
                jsr RestartTimer
                ;do task ********************************************
                ;change state to PUSHBUTTON if fire pressed
                jsr GetFirePressed
                bne .treatflowstateanimation_continue
                inc COIN
                jsr ClearScreen
                jsr DoCommandPrintTopLines
                ;trigger sound: 'Get ready'
                jsr TriggerSoundGetReady
                ;change state
                lda #FLOW_STATE_PUSHBUTTON
                sta FLOWSTATE
                rts
.treatflowstateanimation_continue
                ;draw shape
                lda #$0F
                sta CURRENTSHAPEXPOS
                ;based on frame index
                ldy TITLEBIRDFRAMEINDEX
                lda birdAnimationOffsetTable,Y
                sta CURRENTSHAPEYPOS
                ;based on shape number
                ldx birdAnimationFrameTable,Y
                lda birdShapeAddrTableLow,X
                sta STRINGL
                lda birdShapeAddrTableHigh,X
                sta STRINGH
                lda birdColorAddrTableLow,X
                sta TEXTCOLORL
                lda birdColorAddrTableHigh,X
                sta TEXTCOLORH
                lda #$02
                sta BLOCKWIDTH
                lda #$07
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH-2
                sta MAXSCREENX
                jsr DrawShape
                inc TITLEBIRDFRAMEINDEX
                lda TITLEBIRDFRAMEINDEX
                cmp #85
                bne .treatflowstateanimation_exit
.treatflowstateanimation_changestate
                ;change state to: TITLE
                lda #FLOW_STATE_TITLE
                sta FLOWSTATE
.treatflowstateanimation_exit
                rts

;********************************************************************
; Subroutine:   TreatFlowStatePushButton
;********************************************************************
TreatFlowStatePushButton
                ;****************************
                ;* FLOWSTATE:               *
                ;*  FLOW_STATE_PUSHBUTTON   *
                ;*                          *
                ;****************************
                lda COIN
                cmp #01
                bne .treatflowstatepushbutton_2
                jsr DrawPushButton
                jmp .treatflowstatepushbutton_check
.treatflowstatepushbutton_2
                jsr DrawPushButton2
.treatflowstatepushbutton_check
                jsr GetFirePressed
                bne .treatflowstatepushbutton_exit
                ;use one coin
                dec COIN
                ;change state to: INIT
                lda #FLOW_STATE_INIT
                sta FLOWSTATE
.treatflowstatepushbutton_exit
                rts

;********************************************************************
; Subroutine:   TreatFlowStateInit
;********************************************************************
TreatFlowStateInit
                ;****************************
                ;* FLOWSTATE:               *
                ;*  FLOW_STATE_INIT         *
                ;*                          *
                ;****************************
                lda #FALSE
                sta FLOWTITLESTARTED
                ;reset title timer
                ldy #TIMER_0
                jsr ResetTimer
                ;clear screen
                jsr ClearScreen
                ;set charactes to set B
                lda #CHAR_SETB_BASE
                sta CHRSETBASE
                jsr SetCharSet
                ;start background starfield
                jsr DoCommandStartStarfield
                ;level init
                lda #$00
                sta OVERALLGAMEINDEX
                lda #FLOW_WAVE_1
                sta CURRENTWAVE
                lda #FLOW_ALIEN_FORM_A
                sta CURRENTALIENFORM
                jsr FlowLoadCurrentAlienArray
                ;reset alien color set
                lda #COLOR_SET1
                sta ALIENCOLORSET
                jsr LoadAlienColorSet
                ;reset lives and score
                jsr ResetLives
                jsr ResetScore
                ;init game objects
                jsr AliensInit
                jsr BirdsInit
                jsr PlayerInit
                jsr DoCommandPrintTopLines
                lda #$01
                sta DIFFICULTY
                ;initial color for the spiral fill animation
                lda #$00
                sta SPIRALFILLCOLORARRAYINDEX
                ;reset pointer
                lda #<particleExplosionTable
                sta PARTICLEEXPLOSIONPOINTERL
                lda #>particleExplosionTable
                sta PARTICLEEXPLOSIONPOINTERH
                ;start ship timers
                ldy #TIMER_1
                jsr StartTimer
                ldy #TIMER_2
                jsr StartTimer
                ;start alien timers
                ldy #TIMER_6
                jsr StartTimer
                ldy #TIMER_8
                jsr StartTimer
                ;Start tune #4
                jsr TriggerSoundTune4
                ;change state to: MENU
                lda #FLOW_STATE_MENU
                sta FLOWSTATE
                rts

;********************************************************************
; Subroutine:   TreatFlowStateMenu
;********************************************************************
TreatFlowStateMenu
                ;****************************
                ;* FLOWSTATE:               *
                ;*  FLOW_STATE_MENU         *
                ;*                          *
                ;****************************
                ;start the play break *******************************
                ldy #TIMER_7
                lda timerState,Y
                cmp #TIMER_STATE_INACTIVE
                bne .treatflowstatemenu_playbreak
                jsr StartTimer
                jsr PlayerInit
                rts
.treatflowstatemenu_playbreak
                ldy #TIMER_7
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .treatflowstatemenu_end
                jsr ResetTimer
                ;end of play break **********************************
                ;check WAVE number
                lda CURRENTWAVE
                cmp #FLOW_WAVE_3
                beq .treatflowstatemenu_curtainup
                ;WAVE 1 or WAVE 2 ***********************************
                ;start the alien fade in animation timer
                ldy #TIMER_B
                jsr StartTimer
                ;reset alien and bird fade in frame control
                lda #FALSE
                sta FADEINALIENFINISHED
                sta FADEINBIRDFINISHED
                lda #$00
                sta FADEINALIENFRAMEINDEX
                sta FADEINBIRDSFRAMEINDEX
                ;change state to: FADEIN
                lda #FLOW_STATE_FADEIN
                sta FLOWSTATE
                rts
.treatflowstatemenu_curtainup
                ;WAVE 3 *********************************************
                ;top of the screen
                lda #SCREEN_WIDTH
                sta CURRENTBOSSXPOS
                ;assemble the mothership
                lda #$C8
                sta MEMORYRANGE
                lda #<initialBunkerData
                sta SOURCEADDRESS
                lda #>initialBunkerData
                sta SOURCEADDRESS+1
                lda #<mothershipShapeTable
                sta TARGETADDRESS
                lda #>mothershipShapeTable
                sta TARGETADDRESS+1
                jsr CopyNotZeroData
                ;init bunker frame indexes
                lda #SCREEN_HIGHT
                sta MEMORYRANGE
                lda #<initialBunkerFrameIndexTable
                sta SOURCEADDRESS
                lda #>initialBunkerFrameIndexTable
                sta SOURCEADDRESS+1
                lda #<bunkerFrameIndexTable
                sta TARGETADDRESS
                lda #>bunkerFrameIndexTable
                sta TARGETADDRESS+1
                jsr CopyMemory
                ;clear x offset array
                lda #$00
                ldy #$00
.treatflowstatemenu_loop
                sta xOffsetPurLayerArray,Y
                iny
                cpy #$13
                bne .treatflowstatemenu_loop
                ;change state to: CURTAINUP
                lda #FLOW_STATE_CURTAINUP
                sta FLOWSTATE
.treatflowstatemenu_end
                rts

;********************************************************************
; Subroutine:   TreatFlowStateCurtainUp
;********************************************************************
TreatFlowStateCurtainUp
                ;****************************
                ;* FLOWSTATE:               *
                ;*  FLOW_STATE_CURTAINUP    *
                ;*                          *
                ;****************************
                ;draw the mothership
                lda CURRENTBOSSXPOS
                sta CURRENTSHAPEXPOS
                lda initialMothershipYpos
                sta CURRENTSHAPEYPOS
                lda #<mothershipShapeTable
                sta STRINGL
                lda #>mothershipShapeTable
                sta STRINGH
                lda #<mothershipColorTable
                sta TEXTCOLORL
                lda #>mothershipColorTable
                sta TEXTCOLORH
                lda #$0A
                sta BLOCKWIDTH
                lda #$14
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH-3
                sta MAXSCREENX
                jsr DrawShape
                ;adjust speed
                ldy #TIMER_1
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .treatflowstatecurtainup_skip
                jsr RestartTimer
                ;move down
                lda CURRENTBOSSXPOS
                cmp initialMothershipXpos
                beq .treatflowstatecurtainup_fadein
                dec CURRENTBOSSXPOS
.treatflowstatecurtainup_skip
                rts
.treatflowstatecurtainup_fadein
                ;init the bunkers edge X positions
                ldy #$00
.treatflowstatecurtainup_loop
                lda initialBunkersEdgeXArray,Y
                sta currentBunkersEdgeXArray,Y
                iny
                cpy #NO_OF_BUNKER_EDGES
                bne .treatflowstatecurtainup_loop
                ;start the alien fade in animation timer
                ldy #TIMER_B
                jsr StartTimer
                ;reset alien fade in frame control
                lda #FALSE
                sta FADEINALIENFINISHED
                lda #$00
                sta FADEINALIENFRAMEINDEX
                ;change state to: FADEIN
                lda #FLOW_STATE_FADEIN
                sta FLOWSTATE
                rts

;********************************************************************
; Subroutine:   TreatFlowStateFadeIn
;********************************************************************
TreatFlowStateFadeIn
                ;****************************
                ;* FLOWSTATE:               *
                ;*  FLOW_STATE_FADEIN       *
                ;*                          *
                ;****************************
                lda CURRENTWAVE
                cmp #FLOW_WAVE_2
                bne .treatflowstatefadein_wave13
                ;WAVE 2 *********************************************
                lda FADEINBIRDFINISHED
                cmp #TRUE
                bne .treatflowstatefadein_contiuew2
                ;stop the alien fade in animation timer
                ldy #TIMER_B
                jsr ResetTimer
                ;reset
                jsr PlayerReset
                ;start game speed timer
                ldy #TIMER_9
                jsr StartTimer
                ;setup first timer value for birds x animation
                ldy #TIMER_G
                lda #BIRDSX_SPEED_TIMER_VALUE_H
                ldx #BIRDSX_SPEED_TIMER_VALUE_L
                jsr SetTimerValue
                jsr RestartTimer
                ;change state to: ALIVE
                lda #FLOW_STATE_ALIVE
                sta FLOWSTATE
                rts
.treatflowstatefadein_contiuew2
                jsr DrawBirdsFadeIn
                rts

.treatflowstatefadein_wave13
                ;WAVE 1 or WAVE 3 ***********************************
                lda FADEINALIENFINISHED
                cmp #TRUE
                bne .treatflowstatefadein_contiue
                ;stop the alien fade in animation timer
                ldy #TIMER_B
                jsr ResetTimer
                ;reset
                jsr PlayerReset
                ;start alien attack interval timer
                ldy #TIMER_C
                jsr StartTimer
                ;start alien attack movement timer
                ldy #TIMER_D
                jsr StartTimer
                ;start mothership movement timer
                ldy #TIMER_E
                jsr StartTimer
                ;start alien pilot animation timer
                ldy #TIMER_F
                jsr StartTimer
                ;start game speed timer
                ldy #TIMER_9
                jsr StartTimer
                ;change state to: PICKUP
                lda #ALIEN_STATE_PICKUP
                sta alienState
                ;change state to: ALIVE
                lda #FLOW_STATE_ALIVE
                sta FLOWSTATE
                rts
.treatflowstatefadein_contiue
                jsr DrawAliensFadeIn
                rts

;********************************************************************
; Subroutine:   TreatFlowStateAlive
;********************************************************************
TreatFlowStateAlive
                ;****************************
                ;* FLOWSTATE:               *
                ;*  FLOW_STATE_ALIVE        *
                ;*                          *
                ;****************************
                ;adjust the speed for alien movements
                jsr HandleGameSpeed
                ;check if all aliens are destroyed
                jsr CheckVictory
                lda VICTORYSTATE
                cmp #TRUE
                bne .treatflowstatealive_exit
                ;stop alien attack interval timer
                ldy #TIMER_C
                jsr ResetTimer
                ;stop alien attack movement timer
                ldy #TIMER_D
                jsr ResetTimer
                ;stops in flight bullets from scoring
                jsr BulletsReset
                ;check for pending explosion
                jsr AlienExplosionsPending
                lda EXPLOSIONSPENDING
                cmp #TRUE
                beq .treatflowstatealive_exit
                ;do the animation
                jsr DrawAnimation
                ;go to next formation *******************************
                jsr GoToNextLevel
                jsr ClearScreen
                jsr PlayerInit
                jsr AliensInit
                jsr BirdsInit
                jsr DeactivateHiScoreFlash
                jsr DoCommandPrintTopLines
                lda #FLOW_STATE_MENU
                sta FLOWSTATE
.treatflowstatealive_exit
                rts

;********************************************************************
; Subroutine:   TreatFlowStateDying
;********************************************************************
TreatFlowStateDying
                ;****************************
                ;* FLOWSTATE:               *
                ;*  FLOW_STATE_DYING        *
                ;*                          *
                ;****************************
                lda PLAYER1LIVES
                bne .treatflowstatedying_haslives
                ;reset wave
                lda #FLOW_WAVE_1
                sta CURRENTWAVE
                jsr ClearScreen
                ;stop alien attac interval timer
                ldy #TIMER_C
                jsr ResetTimer
                ;stop alien attack movement timer
                ldy #TIMER_D
                jsr ResetTimer
                ;load default values for alien movement speed
                lda #$00
                sta GAMESPEEDINDEX
                ldy #TIMER_6
                lda #ALIEN_SPEED_TIMER_VALUE_H
                ldx #ALIEN_SPEED_TIMER_VALUE_L
                jsr SetTimerValue
                ldy #TIMER_D
                lda #ALIEN_ATTACK_SPEED_TIMER_VALUE_H
                ldx #ALIEN_ATTACK_SPEED_TIMER_VALUE_L
                jsr SetTimerValue
                ;change state to: GAMEOVER
                lda #FLOW_STATE_GAMEOVER
                sta FLOWSTATE
                rts
.treatflowstatedying_haslives
                ;losing one live
                ;flash score
                jsr ActivateScoreFlash
                jsr ScoreDisplay
                ;stop starfield background
                lda #FALSE
                sta STARSALLOWED
                ;stop alien attac interval timer
                ldy #TIMER_C
                jsr ResetTimer
                ;stop alien attack movement timer
                ldy #TIMER_D
                jsr ResetTimer
                ;start the play break *******************************
                ldy #TIMER_7
                lda timerState,Y
                cmp #TIMER_STATE_INACTIVE
                bne .treatflowstatedying_playbreak
                jsr StartTimer
                rts
.treatflowstatedying_playbreak
                ldy #TIMER_7
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .treatflowstatedying_end
                jsr ResetTimer
                ;end of play break **********************************
                ;deactivate flash
                jsr DeactivateScoreFlash
                jsr ScoreDisplay
                ;start starfield background
                lda #TRUE
                sta STARSALLOWED
                ;start the alien fade in animation timer
                ldy #TIMER_B
                jsr StartTimer
                ;change state to: FADEIN
                lda #FLOW_STATE_FADEIN
                sta FLOWSTATE
                ;init aliens or birds
                lda CURRENTWAVE
                cmp #FLOW_WAVE_2
                bne .treatflowstatedying_wave13
                ;WAVE 2 *********************************************
                ;delete all birds from screen
                jsr DeleteAllBirdsFromScreen
                ;reset existing birds
                jsr BirdsIngameInit
                ;reset bird fade in frame control
                lda #FALSE
                sta FADEINBIRDFINISHED
                lda #$00
                sta FADEINBIRDSFRAMEINDEX
.treatflowstatedying_wave13
                ;WAVE 1 or 3 ****************************************
                ;delete all aliens from screen
                jsr DeleteAllAliensFromScreen
                ;reset existing aliens
                jsr AliensInitPart1
                jsr AliensIngameInit
                ;reset alien fade in frame control
                lda #FALSE
                sta FADEINALIENFINISHED
                lda #$00
                sta FADEINALIENFRAMEINDEX
.treatflowstatedying_end
                rts

;********************************************************************
; Subroutine:   TreatFlowStateGameOver
;********************************************************************
TreatFlowStateGameOver
                ;****************************
                ;* FLOWSTATE:               *
                ;*  FLOW_STATE_GAMEOVER     *
                ;*                          *
                ;****************************
                ;check for pending explosion
                lda particleExplosionTriggered
                cmp #FALSE
                bne .treatflowstategameover_end
                ;clear screen and change state to: PUSHBUTTON if zero coins
                lda COIN
                cmp #$00
                beq .treatflowstategameover_gameover
                jsr DeactivateHiScoreFlash
                jsr ClearScreen
                jsr DoCommandPrintTopLines
                ;trigger sound: 'Get ready'
                jsr TriggerSoundGetReady
                ;change state to: PUSHBUTTON
                lda #FLOW_STATE_PUSHBUTTON
                sta FLOWSTATE
                rts
.treatflowstategameover_gameover
                lda STARSALLOWED    ;workaround
                beq .treatflowstategameover_skip
                ;set charactes to set A
                lda #CHAR_SETA_BASE
                sta CHRSETBASE
                jsr SetCharSet
                ;GAME OVER
                jsr DoCommandPrintTopLines
                jsr DrawGameOver
                jsr DoCommandPrintBottomLines
                ;stop starfield background
                lda #FALSE
                sta STARSALLOWED
.treatflowstategameover_skip
                ;start the play break *******************************
                ldy #TIMER_7
                lda timerState,Y
                cmp #TIMER_STATE_INACTIVE
                bne .treatflowstategameover_playbreak
                jsr StartTimer
                ;Start tune #2
                jsr TriggerSoundTune2
                rts
.treatflowstategameover_playbreak
                ldy #TIMER_7
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .treatflowstategameover_end
                jsr ResetTimer
                ;end of play break **********************************
                jsr ClearScreen
                jsr PlayerInit
                jsr AliensInit
                jsr BirdsInit
                jsr DeactivateHiScoreFlash
                jsr DoCommandPrintTopLines
                ;change state to: TITLE
                lda #FLOW_STATE_TITLE
                sta FLOWSTATE
.treatflowstategameover_end
                rts

;********************************************************************
; Subroutine:   TreatFlowStateTheEnd
;********************************************************************
TreatFlowStateTheEnd
                ;****************************
                ;* FLOWSTATE:               *
                ;*  FLOW_STATE_THEEND       *
                ;*                          *
                ;****************************
                ;start the play break *******************************
                ldy #TIMER_7
                lda timerState,Y
                cmp #TIMER_STATE_INACTIVE
                bne .treatflowstatetheend_playbreak
                jsr StartTimer
                rts
.treatflowstatetheend_playbreak
                ldy #TIMER_7
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .treatflowstatetheend_exit
                jsr ResetTimer
                ;end of play break **********************************
                ;go to next formation *******************************
                jsr GoToNextLevel
                jsr ClearScreen
                jsr PlayerInit
                jsr AliensInit
                jsr BirdsInit
                jsr DeactivateHiScoreFlash
                jsr DoCommandPrintTopLines
                ;change state to: Menu
                lda #FLOW_STATE_MENU
                sta FLOWSTATE
.treatflowstatetheend_exit
                rts

;********************************************************************
; Subroutine:   ScoreDisplay
;********************************************************************
ScoreDisplay
                ;draw PLAYER1SCOREHIGH
                lda #SCORE_NUM_X_POS
                sta CURRENTXPOS
                lda #SCORE_NUM_Y_POS
                sta CURRENTYPOS
                lda PLAYER1SCOREHIGH
                sta CURRENTDECNUMBER
                lda scoreColor
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                ;draw PLAYER1SCOREMID
                lda #SCORE_NUM_Y_POS+2
                sta CURRENTYPOS
                lda PLAYER1SCOREMID
                sta CURRENTDECNUMBER
                lda scoreColor
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                ;draw PLAYER1SCORELOW
                lda #SCORE_NUM_Y_POS+4
                sta CURRENTYPOS
                lda PLAYER1SCORELOW
                sta CURRENTDECNUMBER
                lda scoreColor
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                rts

;********************************************************************
; Subroutine:   Score2Display
;********************************************************************
Score2Display
                ;draw high part of player 2 score
                lda #SCORE2_NUM_X_POS
                sta CURRENTXPOS
                lda #SCORE2_NUM_Y_POS
                sta CURRENTYPOS
                lda PLAYER2SCOREHIGH
                sta CURRENTDECNUMBER
                lda scoreColor
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                ;draw mid part of player 2 score
                lda #SCORE2_NUM_Y_POS+2
                sta CURRENTYPOS
                lda PLAYER2SCOREMID
                sta CURRENTDECNUMBER
                lda scoreColor
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                ;draw low part of player 2 score
                lda #SCORE2_NUM_Y_POS+4
                sta CURRENTYPOS
                lda PLAYER2SCORELOW
                sta CURRENTDECNUMBER
                lda scoreColor
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                rts

;********************************************************************
; Subroutine:   LivesDisplay
;********************************************************************
LivesDisplay
                ;draw PLAYER1LIVES
                lda #LIVES_NUM_X_POS
                sta CURRENTXPOS
                lda #LIVES_NUM_Y_POS
                sta CURRENTYPOS
                lda PLAYER1LIVES
                sta CURRENTDECNUMBER
                lda #SCCO
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                rts

;********************************************************************
; Subroutine:   Lives2Display
;********************************************************************
Lives2Display
                ;draw lives
                lda #LIVES2_NUM_X_POS
                sta CURRENTXPOS
                lda #LIVES2_NUM_Y_POS
                sta CURRENTYPOS
                lda PLAYER2LIVES
                sta CURRENTDECNUMBER
                lda #SCCO
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                rts

;********************************************************************
; Subroutine:   CoinDisplay
;********************************************************************
CoinDisplay
                ;draw coins
                lda #COIN_NUM_X_POS
                sta CURRENTXPOS
                lda #COIN_NUM_Y_POS
                sta CURRENTYPOS
                lda COIN
                sta CURRENTDECNUMBER
                lda #SCCO
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                rts

;********************************************************************
; Subroutine:   HiScoreDisplay
;********************************************************************
HiScoreDisplay
                ;draw high part of high score
                lda #HI_NUM_X_POS
                sta CURRENTXPOS
                lda #HI_NUM_Y_POS
                sta CURRENTYPOS
                lda HIGHSCOREHIGH
                sta CURRENTDECNUMBER
                lda hiScoreColor
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                ;draw mid part of high score
                lda #HI_NUM_Y_POS+2
                sta CURRENTYPOS
                lda HIGHSCOREMID
                sta CURRENTDECNUMBER
                lda hiScoreColor
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                ;draw low part of high score
                lda #HI_NUM_Y_POS+4
                sta CURRENTYPOS
                lda HIGHSCORELOW
                sta CURRENTDECNUMBER
                lda hiScoreColor
                sta CURRENTCOLOR
                jsr DrawDecimalRotated
                rts

;********************************************************************
; Subroutine:   IncreaseScoreLow
;********************************************************************
IncreaseScoreLow
;used for 20, 40, 50 or 80 points
                sed
                clc
                lda VALUETOSCORE
                adc PLAYER1SCORELOW
                sta PLAYER1SCORELOW
                lda PLAYER1SCOREMID
                adc #00
                sta PLAYER1SCOREMID
                lda PLAYER1SCOREHIGH
                adc #00
                sta PLAYER1SCOREHIGH
                cld
                jsr CheckForP1Bonus
                jsr ScoreDisplay
                rts

;********************************************************************
; Subroutine:   IncreaseScoreHigh
;********************************************************************
IncreaseScoreHigh
;used for 100, 200, 800, 1000, 9000 points
                sed
                clc
                lda VALUETOSCORE
                adc PLAYER1SCOREMID
                sta PLAYER1SCOREMID
                lda PLAYER1SCOREHIGH
                adc #00
                sta PLAYER1SCOREHIGH
                cld
                jsr CheckForP1Bonus
                jsr ScoreDisplay
                rts

;********************************************************************
; Subroutine:   CheckForP1Bonus
;********************************************************************
CheckForP1Bonus
                lda BONUSP1FLAG         ;bonus just for one time
                cmp #FALSE
                bne .checkforp1bonus_nobonus
                lda PLAYER1SCOREMID
                cmp #BONUS_ONE_LIVE
                bcc .checkforp1bonus_nobonus
                inc PLAYER1LIVES
                jsr LivesDisplay
                lda #TRUE
                sta BONUSP1FLAG
                jsr TriggerSoundTune1   ;TriggerSoundBonus
.checkforp1bonus_nobonus
                rts

;********************************************************************
; Subroutine:   FlowLoadCurrentAlienArray
;********************************************************************
FlowLoadCurrentAlienArray
                ;for the x and y positions of 16 aliens
                lda #$20
                sta MEMORYRANGE
                ldx CURRENTALIENFORM
                lda alienFormationTableLow,X
                sta SOURCEADDRESS
                lda alienFormationTableHigh,X
                sta SOURCEADDRESS+1
                lda #<aliensInitialXArray
                sta TARGETADDRESS
                lda #>aliensInitialXArray
                sta TARGETADDRESS+1
                jsr CopyMemory
                rts

;********************************************************************
; Subroutine:   FlowLoadCurrentBirdArray
;********************************************************************
FlowLoadCurrentBirdArray
                ;for the x positions and the y offsets of 8 birds
                lda #$18
                sta MEMORYRANGE
                ldx CURRENTBIRDFORM
                lda birdFormationTableLow,X
                sta SOURCEADDRESS
                lda birdFormationTableHigh,X
                sta SOURCEADDRESS+1
                lda #<birdInitialXArray
                sta TARGETADDRESS
                lda #>birdInitialXArray
                sta TARGETADDRESS+1
                jsr CopyMemory
                rts

;********************************************************************
; Subroutine:   FlowLoadCurrentBossArray
;********************************************************************
FlowLoadCurrentBossArray
                ;for the x and y positions of 16 aliens
                lda #$20
                sta MEMORYRANGE
                ldx CURRENTBOSSFORM
                lda alienFormationTableLow,X
                sta SOURCEADDRESS
                lda alienFormationTableHigh,X
                sta SOURCEADDRESS+1
                lda #<aliensInitialXArray
                sta TARGETADDRESS
                lda #>aliensInitialXArray
                sta TARGETADDRESS+1
                jsr CopyMemory
                rts

;********************************************************************
; Subroutine:   ResetScore
;********************************************************************
ResetScore
                lda #$00
                sta PLAYER1SCORELOW
                sta PLAYER1SCOREMID
                sta PLAYER1SCOREHIGH
                jsr ScoreDisplay
                rts

;********************************************************************
; Subroutine:   ResetLives
;********************************************************************
ResetLives
                lda #FLOW_NUM_LIVES
                sta PLAYER1LIVES
                jsr LivesDisplay
                rts

;********************************************************************
; Subroutine:   UpdateHiScore
;********************************************************************
UpdateHiScore
                ;compare multi-byte numbers
                lda PLAYER1SCORELOW
                cmp HIGHSCORELOW
                lda PLAYER1SCOREMID
                sbc HIGHSCOREMID
                lda PLAYER1SCOREHIGH
                sbc HIGHSCOREHIGH
                bcc .updatehiscore_exit
                jsr ActivateHiScoreFlash
                lda PLAYER1SCORELOW
                sta HIGHSCORELOW
                lda PLAYER1SCOREMID
                sta HIGHSCOREMID
                lda PLAYER1SCOREHIGH
                sta HIGHSCOREHIGH
                jsr HiScoreDisplay
.updatehiscore_exit
                rts

;********************************************************************
; Subroutine:   DrawPushButton
;********************************************************************
DrawPushButton
                ;'PUSH'
                lda #START_TEXT_X_POS
                sta CURRENTXPOS
                lda #START_TEXT_Y_POS
                sta CURRENTYPOS
                ;clear the text every 5th frame
                lda FRAMECOUNTER
                and #FRAME_MASK
                bne .drawpushbutton_oddframe
                lda #<flowStart1TextClr
                sta STRINGL
                lda #>flowStart1TextClr
                sta STRINGH
                jmp .drawpushbutton_continue
.drawpushbutton_oddframe
                lda #<flowStart1Text
                sta STRINGL
                lda #>flowStart1Text
                sta STRINGH
.drawpushbutton_continue
                lda #<flowStart1TextColor
                sta TEXTCOLORL
                lda #>flowStart1TextColor
                sta TEXTCOLORH
                jsr DrawTextRotated
                ;'ONLY 1PLAYER BUTTON'
                lda #START_TEXT_X_POS-4
                sta CURRENTXPOS
                lda #$03
                sta CURRENTYPOS
                ;clear the text every 3rd frame
                lda FRAMECOUNTER
                and #FRAME_MASK
                bne .drawpushbutton_oddframe2
                lda #<flowStart2TextClr
                sta STRINGL
                lda #>flowStart2TextClr
                sta STRINGH
                jmp .drawpushbutton_continue2
.drawpushbutton_oddframe2
                lda #<flowStart2Text
                sta STRINGL
                lda #>flowStart2Text
                sta STRINGH
.drawpushbutton_continue2
                lda #<flowStart2TextColor
                sta TEXTCOLORL
                lda #>flowStart2TextColor
                sta TEXTCOLORH
                jsr DrawTextRotated
                rts

;********************************************************************
; Subroutine:   DrawPushButton2
;********************************************************************
DrawPushButton2
                ;'PUSH'
                lda #START_TEXT_X_POS
                sta CURRENTXPOS
                lda #START_TEXT_Y_POS
                sta CURRENTYPOS
                ;clear the text every 5th frame
                lda FRAMECOUNTER
                and #FRAME_MASK
                bne .drawpushbutton2_oddframe
                lda #<flowStart1TextClr
                sta STRINGL
                lda #>flowStart1TextClr
                sta STRINGH
                jmp .drawpushbutton2_continue
.drawpushbutton2_oddframe
                lda #<flowStart1Text
                sta STRINGL
                lda #>flowStart1Text
                sta STRINGH
.drawpushbutton2_continue
                lda #<flowStart1TextColor
                sta TEXTCOLORL
                lda #>flowStart1TextColor
                sta TEXTCOLORH
                jsr DrawTextRotated
                ;'1 OR 2PLAYERS BUTTON'
                lda #START_TEXT_X_POS-4
                sta CURRENTXPOS
                lda #$02
                sta CURRENTYPOS
                ;clear the text every 5th frame
                lda FRAMECOUNTER
                and #FRAME_MASK
                bne .drawpushbutton2_oddframe2
                lda #<flowStart3TextClr
                sta STRINGL
                lda #>flowStart3TextClr
                sta STRINGH
                jmp .drawpushbutton2_continue2
.drawpushbutton2_oddframe2
                lda #<flowStart3Text
                sta STRINGL
                lda #>flowStart3Text
                sta STRINGH
.drawpushbutton2_continue2
                lda #<flowStart3TextColor
                sta TEXTCOLORL
                lda #>flowStart3TextColor
                sta TEXTCOLORH
                jsr DrawTextRotated
                rts

;********************************************************************
; Subroutine:   DrawGameOver
;********************************************************************
DrawGameOver
                ;'GAME  OVER'
                lda #GAMEOVER_TEXT_X_POS
                sta CURRENTXPOS
                lda #GAMEOVER_TEXT_Y_POS
                sta CURRENTYPOS
                lda #<flowEndText
                sta STRINGL
                lda #>flowEndText
                sta STRINGH
                lda #<flowEndTextColor
                sta TEXTCOLORL
                lda #>flowEndTextColor
                sta TEXTCOLORH
                jsr DrawTextRotated
                rts

;********************************************************************
; Subroutine:   WaitForFirePressed
;********************************************************************
WaitForFirePressed
                ;wait for fire pressed
.waitforfirepressed_nofire
                jsr InputUpdate
                jsr GetFirePressed
                bne .waitforfirepressed_nofire
                rts

;********************************************************************
; Subroutine:   IntroCommandCase
;********************************************************************
IntroCommandCase
;if CMD_WAIT
.introcommandcase_wait
                lda TITLECMD
                cmp #CMD_WAIT
                bne .introcommandcase_printtoplines
                ;just for update of the timer value
                jmp .introcommandcase_inc
;if CMD_PRINT_TOP_LINES
.introcommandcase_printtoplines
                lda TITLECMD
                cmp #CMD_PRINT_TOP_LINES
                bne .introcommandcase_printbottomlines
                jsr DoCommandPrintTopLines
                jmp .introcommandcase_inc
;if CMD_PRINT_BOTTOM_LINES
.introcommandcase_printbottomlines
                lda TITLECMD
                cmp #CMD_PRINT_BOTTOM_LINES
                bne .introcommandcase_tostateslowprint
                jsr DoCommandPrintBottomLines
                jmp .introcommandcase_inc
;if CMD_TO_STATE_SLOWPRINT
.introcommandcase_tostateslowprint
                lda TITLECMD
                cmp #CMD_TO_STATE_SLOWPRINT
                bne .introcommandcase_draw4shapes
                jsr DoCommandToStateSlowPrint
                jmp .introcommandcase_inc
;if CMD_DRAW_4_SHAPES
.introcommandcase_draw4shapes
                lda TITLECMD
                cmp #CMD_DRAW_4_SHAPES
                bne .introcommandcase_clearscreen
                jsr DoCommandDraw4Shapes
                jmp .introcommandcase_inc
;if CMD_CLEAR_SCREEN
.introcommandcase_clearscreen
                lda TITLECMD
                cmp #CMD_CLEAR_SCREEN
                bne .introcommandcase_startstarfield
                jsr ClearScreen
                jmp .introcommandcase_inc
;if CMD_START_STARFIELD
.introcommandcase_startstarfield
                lda TITLECMD
                cmp #CMD_START_STARFIELD
                bne .introcommandcase_stopstarfield
                jsr DoCommandStartStarfield
                jmp .introcommandcase_inc
;if CMD_STOP_STARFIELD
.introcommandcase_stopstarfield
                lda TITLECMD
                cmp #CMD_STOP_STARFIELD
                bne .introcommandcase_tostateanimation
                jsr DoCommandStopStarfield
                jmp .introcommandcase_inc
;if CMD_TO_STATE_ANIMATION
.introcommandcase_tostateanimation
                lda TITLECMD
                cmp #CMD_TO_STATE_ANIMATION
                bne .introcommandcase_resetpointers
                jsr DoCommandToStateAnimation
                jmp .introcommandcase_inc
;if CMD_RESET_POINTERS
.introcommandcase_resetpointers
                lda TITLECMD
                cmp #CMD_RESET_POINTERS
                bne .introcommandcase_exit
                jsr DoCommandResetPointers
                jsr LoadTitleColorSet
                rts
.introcommandcase_inc
                jsr IncExecutionTablePointer
.introcommandcase_exit
                rts

;********************************************************************
; Subroutine:   DoCommandPrintTopLines
;********************************************************************
DoCommandPrintTopLines
                ;draw the static text
                lda #$25
                sta CURRENTSHAPEXPOS
                lda #$00
                sta CURRENTSHAPEYPOS
                lda #<topText
                sta STRINGL
                lda #>topText
                sta STRINGH
                lda #<topTextColor
                sta TEXTCOLORL
                lda #>topTextColor
                sta TEXTCOLORH
                lda #$03
                sta BLOCKWIDTH
                lda #$19
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH
                sta MAXSCREENX
                jsr DrawShape
                ;display the values
                jsr ScoreDisplay
                jsr LivesDisplay
                jsr HiScoreDisplay
                jsr Score2Display
                jsr Lives2Display
                jsr CoinDisplay
                rts

;********************************************************************
; Subroutine:   DoCommandPrintBottomLines
;********************************************************************
DoCommandPrintBottomLines
                lda #$00
                sta CURRENTSHAPEXPOS
                sta CURRENTSHAPEYPOS
                lda #<introBottomText
                sta STRINGL
                lda #>introBottomText
                sta STRINGH
                lda #<introBottomTextColor
                sta TEXTCOLORL
                lda #>introBottomTextColor
                sta TEXTCOLORH
                lda #$03
                sta BLOCKWIDTH
                lda #$19
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH
                sta MAXSCREENX
                jsr DrawShape
                rts

;********************************************************************
; Subroutine:   DoCommandToStateSlowPrint
;********************************************************************
DoCommandToStateSlowPrint
                ;change state to: SLOWPRINT
                lda #FLOW_STATE_SLOWPRINT
                sta FLOWSTATE
                rts

;********************************************************************
; Subroutine:   DoCommandDraw4Shapes
;********************************************************************
DoCommandDraw4Shapes
                lda #$08
                sta CURRENTSHAPEXPOS
                lda #$00
                sta CURRENTSHAPEYPOS
                lda #<the4Shapes
                sta STRINGL
                lda #>the4Shapes
                sta STRINGH
                lda #<the4ShapesColor
                sta TEXTCOLORL
                lda #>the4ShapesColor
                sta TEXTCOLORH
                lda #$0A
                sta BLOCKWIDTH
                lda #$06
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH-2
                sta MAXSCREENX
                jsr DrawShape
                rts

;********************************************************************
; Subroutine:   DoCommandStartStarfield
;********************************************************************
DoCommandStartStarfield
                ;start starfield background
                lda #TRUE
                sta STARSALLOWED
                rts

;********************************************************************
; Subroutine:   DoCommandStopStarfield
;********************************************************************
DoCommandStopStarfield
                ;stop starfield background
                lda #FALSE
                sta STARSALLOWED
                rts

;********************************************************************
; Subroutine:   DoCommandToStateAnimation
;********************************************************************
DoCommandToStateAnimation
                ;change state to: ANIMATION
                lda #FLOW_STATE_ANIMATION
                sta FLOWSTATE
                rts

;********************************************************************
; Subroutine:   DoCommandResetPointers
;********************************************************************
DoCommandResetPointers
                ;reset pointer for: introExecutionTable
                lda #<introExecutionTable
                sta TITLEEXECPOINTER
                lda #>introExecutionTable
                sta TITLEEXECPOINTER+1
                ;reset pointer for title character table
                lda #<gameIntroPage1L1
                sta TITLECHRPOINTER
                lda #>gameIntroPage1L1
                sta TITLECHRPOINTER+1
                ;reset pointer for title color table
                lda #<gameIntroPage1L1Color
                sta TITLECOLORPOINTER
                lda #>gameIntroPage1L1Color
                sta TITLECOLORPOINTER+1
                ;reset index for text lines
                lda #$00
                sta TITLECHRLINEINDEX
                ;reset index for bird frames
                sta TITLEBIRDFRAMEINDEX
                rts

;********************************************************************
; Subroutine:   IncExecutionTablePointer
;********************************************************************
IncExecutionTablePointer
                clc
                lda TITLEEXECPOINTER
                adc #$02                    ;number of fields
                sta TITLEEXECPOINTER
                lda TITLEEXECPOINTER+1
                adc #$00
                sta TITLEEXECPOINTER+1
                rts

;********************************************************************
; Subroutine:   IncTitleLinePointer
;********************************************************************
IncTitleLinePointer
                ;update the pointer for character lines
                clc
                lda TITLECHRPOINTER
                adc #$19                    ;number of bytes for one line
                sta TITLECHRPOINTER
                lda TITLECHRPOINTER+1
                adc #$00
                sta TITLECHRPOINTER+1
                ;update the pointer for color lines
                clc
                lda TITLECOLORPOINTER
                adc #$19                    ;number of bytes for one line
                sta TITLECOLORPOINTER
                lda TITLECOLORPOINTER+1
                adc #$00
                sta TITLECOLORPOINTER+1
                ;increment the counter for the lines
                inc TITLECHRLINEINDEX
                rts

;********************************************************************
; Subroutine:   GetIntroExecutionCommand
;********************************************************************
GetIntroExecutionCommand
                ldy #$00
                lda (TITLEEXECPOINTER),Y
                sta TITLECMD
                rts

;********************************************************************
; Subroutine:   GetIntroExecutionTimer
;********************************************************************
GetIntroExecutionTimer
                ldy #$01
                lda (TITLEEXECPOINTER),Y
                sta TITLETIMER
                rts

;********************************************************************
; Subroutine:   HandleGameSpeed
;********************************************************************
HandleGameSpeed
                ldy #TIMER_9
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .handlegamespeed_notask
                jsr RestartTimer
                ;do task ********************************************
                lda GAMESPEEDINDEX
                cmp #$07
                beq .handlegamespeed_notask
                inc GAMESPEEDINDEX
                ;set the new timer values
                ldy #TIMER_6
                ldx GAMESPEEDINDEX
                lda alienSpeedTable6,X
                tax
                lda #$00
                jsr SetTimerValue
                jsr RestartTimer
                ldy #TIMER_D
                ldx GAMESPEEDINDEX
                lda alienSpeedTableD,X
                tax
                lda #$00
                jsr SetTimerValue
                jsr RestartTimer
.handlegamespeed_notask
                rts

;********************************************************************
; Subroutine:   GoToNextLevel
;********************************************************************
GoToNextLevel
                ;increment over all game flow table index
                inc OVERALLGAMEINDEX
                lda OVERALLGAMEINDEX
                cmp #LENGHT_OF_OVERALLGAMEFLOWTABLE
                bne .gotonextlevel_skip
                lda #$00
                sta OVERALLGAMEINDEX
.gotonextlevel_skip
                ;get/set current wave number
                ldy OVERALLGAMEINDEX
                lda overAllGameWaveTable,Y
                sta CURRENTWAVE
                cmp #FLOW_WAVE_2
                bne .gotonextlevel_wave1or3
                ;get/set current form type for birds
                lda overAllGameFormTable,Y
                sta CURRENTBIRDFORM
                jsr FlowLoadCurrentBirdArray
                ;get/set color set
                ldy OVERALLGAMEINDEX
                lda overAllGameColorSetTable,Y
                sta TITLECOLORSET
                jsr LoadTitleColorSet
                jmp .gotonextlevel_next
.gotonextlevel_wave1or3
                ;get/set current form type for aliens
                lda overAllGameFormTable,Y
                sta CURRENTALIENFORM
                jsr FlowLoadCurrentAlienArray
                ;get/set color set
                ldy OVERALLGAMEINDEX
                lda overAllGameColorSetTable,Y
                sta ALIENCOLORSET
                jsr LoadAlienColorSet
.gotonextlevel_next
                ;get/set character set
                ldy OVERALLGAMEINDEX
                lda overAllGameChrSetTable,Y
                sta CHRSETBASE
                jsr SetCharSet
                rts

;********************************************************************
; Subroutine:   CheckVictory
;********************************************************************
CheckVictory
                lda CURRENTWAVE
                cmp #FLOW_WAVE_3
                bne .checkvictory_notw3
                lda #FALSE
                sta VICTORYSTATE
                rts
.checkvictory_notw3
                lda #TRUE
                sta VICTORYSTATE
                lda CURRENTWAVE
                cmp #FLOW_WAVE_2
                bne .checkvictory_notw2
                ldx #$00
.checkvictory_next2
                lda birdFleetArray,X
                cmp #PRESENT
                bne .checkvictory_skip2
                lda #FALSE
                sta VICTORYSTATE
.checkvictory_skip2
                inx
                cpx #BIRDS_MAX
                bne .checkvictory_next2
                rts
.checkvictory_notw2
                ldx #$00
.checkvictory_next
                lda fleetArray,X
                cmp #PRESENT
                bne .checkvictory_skip
                lda #FALSE
                sta VICTORYSTATE
.checkvictory_skip
                inx
                cpx #ALIENS_MAX
                bne .checkvictory_next
.checkvictory_exit
                rts

;********************************************************************
;game flow data *****************************************************
;********************************************************************

scoreColor
                !byte SCCO

hiScoreColor
                !byte SCCO

;game speed table for timer_6 and timer_d
alienSpeedTable6
                !byte $05,$05,$05,$04,$04,$04,$04,$03

alienSpeedTableD
                !byte $04,$04,$04,$03,$03,$03,$03,$02

;********************************************************************
;CharacterSetA used
;slow print game intro page 1
gameIntroPage1L1
                ;'INSERT COIN'
                !byte 042,042,042,042,042,042,009,014,019,005,018,020,042,042,003,015,009,014,042,042,042,042,042,042,042
                ;'* 1 PLAYER 1 COIN'
                !byte 042,042,030,042,033,016,012,001,025,005,018,042,042,042,033,003,015,009,014,042,042,030,042,042,042
                ;'* 2 PLAYERS 2 COINS'
                !byte 042,042,030,042,034,016,012,001,025,005,018,019,042,042,034,003,015,009,014,019,042,030,042,042,042
                ;'SCORE AVERAGE TABLE'
                !byte 042,042,019,003,015,018,005,042,001,022,005,018,001,007,005,042,020,001,002,012,005,042,042,042,042
                ;'20 40 80'
                !byte 042,042,042,042,042,042,042,034,032,042,036,032,042,040,032,042,042,042,042,042,042,042,042,042,042
                ;'200'
                !byte 042,042,042,042,042,042,042,034,032,032,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042
                ;'50 100 ?[100-800]'
                !byte 042,042,042,042,042,042,042,037,032,042,033,032,032,042,031,027,033,032,032,043,040,032,032,028,042
                ;'1000-9000'
                !byte 042,042,042,042,042,042,042,033,032,032,032,043,041,032,032,032,042,042,042,042,042,042,042,042,042
;slow print game intro page 2
gameIntroPage2L1
                ;'PHOENIX'
                !byte 060,061,060,061,060,061,056,057,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,134,135
                !byte 060,061,042,042,042,042,060,061,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,136,137
                !byte 060,061,060,061,060,061,056,057,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042
                !byte 060,061,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042
                !byte 060,061,042,064,042,064,042,064,064,064,042,064,060,061,042,132,133,042,064,042,064,042,064,042,064
                !byte 060,061,042,064,042,064,042,064,042,064,042,064,042,042,042,064,128,042,064,042,064,042,128,042,129
                !byte 060,061,042,064,064,064,042,064,042,064,042,064,132,133,042,064,078,079,064,042,064,042,042,064,042
                !byte 060,061,042,064,042,064,042,064,042,064,042,064,042,042,042,064,042,128,064,042,064,042,129,042,128
                !byte 060,061,042,064,042,064,042,064,064,064,042,064,060,061,042,064,042,062,063,042,064,042,064,042,064
                ;'RECOVERED'
                !byte 042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,018,005,003,015,022,005,018,005,004

gameIntroPage1L1Color
                !byte WHT,WHT,WHT,WHT,WHT,WHT,CYN6,CYN6,CYN6,CYN6,CYN6,CYN6,WHT,WHT,CYN6,CYN6,CYN6,CYN6,WHT,WHT,WHT,WHT,WHT,WHT,WHT
                !byte WHT,WHT,LGRN,WHT,SCCO,CYN6,CYN6,CYN6,CYN6,CYN6,CYN6,WHT,WHT,WHT,SCCO,CYN6,CYN6,CYN6,CYN6,WHT,WHT,LGRN,WHT,WHT,WHT
                !byte WHT,WHT,LGRN,WHT,SCCO,CYN6,CYN6,CYN6,CYN6,CYN6,CYN6,CYN6,WHT,WHT,SCCO,CYN6,CYN6,CYN6,CYN6,CYN6,WHT,LGRN,WHT,WHT,WHT
                !byte WHT,WHT,CYN6,CYN6,CYN6,CYN6,CYN6,WHT,CYN6,CYN6,CYN6,CYN6,CYN6,CYN6,CYN6,WHT,CYN6,CYN6,CYN6,CYN6,CYN6,WHT,WHT,WHT,WHT
                !byte WHT,WHT,WHT,WHT,WHT,WHT,WHT,SCCO,SCCO,WHT,SCCO,SCCO,WHT,SCCO,SCCO,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT
                !byte WHT,WHT,WHT,WHT,WHT,WHT,WHT,SCCO,SCCO,SCCO,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT
                !byte WHT,WHT,WHT,WHT,WHT,WHT,WHT,SCCO,SCCO,WHT,SCCO,SCCO,SCCO,WHT,SCCO,CYN6,SCCO,SCCO,SCCO,SCCO,SCCO,SCCO,SCCO,CYN6,WHT
                !byte WHT,WHT,WHT,WHT,WHT,WHT,WHT,SCCO,SCCO,SCCO,SCCO,SCCO,SCCO,SCCO,SCCO,SCCO,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT

gameIntroPage2L1Color
                !fill 250, 0

gameIntroTextLines
                !byte 31,29,27,21,17,14,11,8
                !byte 27,26,25,24,23,22,21,20,19,17

;1st color set
titleColorSet1
                !byte YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6,YEL6
                !byte PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,PUR,YEL6,YEL6
                !fill 25, YEL6
                !fill 25, PUR
                !fill 25, YEL6
                !fill 25, PUR
                !fill 25, YEL6
                !fill 25, PUR
                !fill 25, YEL6
                !byte WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,YEL6,PUR,YEL6,PUR,YEL6,PUR,YEL6,PUR,YEL6
;2nd color set
titleColorSet2
                !byte LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN,LGRN
                !byte PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,PNK,LGRN,LGRN
                !fill 25, LGRN
                !fill 25, PNK
                !fill 25, LGRN
                !fill 25, PNK
                !fill 25, LGRN
                !fill 25, PNK
                !fill 25, LGRN
                !byte WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,WHT,LGRN,PNK,LGRN,PNK,LGRN,PNK,LGRN,PNK,LGRN

introExecutionTable
                ;     Command:                  timer value for next command:
                !byte CMD_PRINT_TOP_LINES,      $00     ;direct print 'SCORE, LIVES, HI..'
                !byte CMD_PRINT_BOTTOM_LINES,   $00     ;direct print 'SORBAS C..'
                !byte CMD_TO_STATE_SLOWPRINT,   $01     ;change state to:FLOW_STATE_SLOWPRINT
                !byte CMD_WAIT,                 $30     ;just wait
                !byte CMD_DRAW_4_SHAPES,        $50     ;draw the 4 shapes at 'score average table'
                !byte CMD_WAIT,                 $30     ;just wait
                !byte CMD_CLEAR_SCREEN,         $00     ;clear screen
                !byte CMD_PRINT_TOP_LINES,      $00     ;direct print 'SCORE, LIVES, HI..'
                !byte CMD_PRINT_BOTTOM_LINES,   $00     ;direct print 'SORBAS C..'
                !byte CMD_START_STARFIELD,      $50     ;start background startfield
                !byte CMD_TO_STATE_SLOWPRINT,   $01     ;change state to:FLOW_STATE_SLOWPRINT
                !byte CMD_STOP_STARFIELD,       $20     ;stop background startfield
                !byte CMD_TO_STATE_ANIMATION,   $02     ;change state to:FLOW_STATE_ANIMATION
                !byte CMD_WAIT,                 $50     ;just wait
                !byte CMD_CLEAR_SCREEN,         $00     ;clear screen
                !byte CMD_RESET_POINTERS,       $00     ;loop

;the 4 shapes at score average table
the4Shapes
                !byte 000,000,000,000,232,000,000,000,000,000
                !byte 000,000,000,000,233,000,106,104,000,000
                !byte 142,140,000,198,196,000,107,105,000,060
                !byte 143,141,000,199,197,000,118,116,000,061
                !byte 000,000,000,000,234,000,119,117,000,000
                !byte 000,000,000,000,235,000,000,000,000,000

the4ShapesColor
                !fill 60,0

;color set 1
the4ShapesColorSet1
                !byte 000,000,000,000,CYN6,000,000,000,000,000
                !byte 000,000,000,000,CYN6,000,YEL6,PNK,000,000
                !byte PNK,PUR,000,YEL6,CYN6,000,PNK,YEL6,000,YEL6
                !byte PNK,PUR,000,YEL6,CYN6,000,PNK,YEL6,000,YEL6
                !byte 000,000,000,000,CYN6,000,YEL6,PNK,000,000
                !byte 000,000,000,000,CYN6,000,000,000,000,000
;color set 2
the4ShapesColorSet2
                !byte 000,000,000,000,PUR,000,000,000,000,000
                !byte 000,000,000,000,PUR,000,LGRN,PNK,000,000
                !byte PNK,PUR,000,YEL6,PUR,000,PNK,LGRN,000,LGRN
                !byte PNK,PUR,000,YEL6,PUR,000,PNK,LGRN,000,LGRN
                !byte 000,000,000,000,PUR,000,LGRN,PNK,000,000
                !byte 000,000,000,000,PUR,000,000,000,000,000
;"PUSH"
flowStart1Text
                !byte 016,021,019,008
                !byte 0

flowStart1TextClr
                !byte 042,042,042,042
                !byte 0

flowStart1TextColor
                !byte CYN6,CYN6,CYN6,CYN6
                !byte 0

;"ONLY 1PLAYER BUTTON"
flowStart2Text
                !byte 015,014,012,025,042,033,016,012,001,025,005,018,042,002,021,020,020,015,014
                !byte 0

flowStart2TextClr
                !byte 042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042
                !byte 0

flowStart2TextColor
                !byte CYN6,CYN6,CYN6,CYN6,WHT,SCCO,CYN6,CYN6,CYN6,CYN6,CYN6,CYN6,WHT,CYN6,CYN6,CYN6,CYN6,CYN6,CYN6
                !byte 0

;"1 OR 2PLAYERS BUTTON"
flowStart3Text
                !byte 033,042,015,018,042,034,016,012,001,025,005,018,019,042,002,021,020,020,015,014
                !byte 0

flowStart3TextClr
                !byte 042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042,042
                !byte 0

flowStart3TextColor
                !byte SCCO,WHT,CYN6,CYN6,WHT,SCCO,CYN6,CYN6,CYN6,CYN6,CYN6,CYN6,CYN6,WHT,CYN6,CYN6,CYN6,CYN6,CYN6,CYN6
                !byte 0

;"GAME OVER"
flowEndText     
                !byte 007,001,013,005,042,042,015,022,005,018
                !byte 0

flowEndTextColor
                !byte CYNF,CYNF,CYNF,CYNF,WHT,WHT,CYNF,CYNF,CYNF,CYNF
                !byte 0

;********************************************************************
;'SCORE1  HI-SCORE   SCORE2'
;'                         '
;'  X      COIN        X   '
topText
                !byte 000,042,019
                !byte 000,042,003
                !byte 047,042,015
                !byte 042,042,018
                !byte 042,042,005
                !byte 000,042,033
                !byte 000,000,000
                !byte 000,000,000
                !byte 000,000,008
                !byte 003,042,009
                !byte 015,042,043
                !byte 009,042,019
                !byte 014,042,003
                !byte 042,042,015
                !byte 042,042,018
                !byte 000,000,005
                !byte 000,000,000
                !byte 000,000,000
                !byte 000,000,000
                !byte 000,042,019
                !byte 000,042,003
                !byte 047,042,015
                !byte 042,042,018
                !byte 042,042,005
                !byte 000,042,034

topTextColor
                !byte 000,SCCO,CYN6
                !byte 000,SCCO,CYN6
                !byte PNK,SCCO,CYN6
                !byte SCCO,SCCO,CYN6
                !byte SCCO,SCCO,CYN6
                !byte 000,SCCO,SCCO
                !byte 000,000,000
                !byte 000,000,000
                !byte 000,000,CYN6
                !byte CYN6,SCCO,CYN6
                !byte CYN6,SCCO,SCCO
                !byte CYN6,SCCO,CYN6
                !byte CYN6,SCCO,CYN6
                !byte SCCO,SCCO,CYN6
                !byte SCCO,SCCO,CYN6
                !byte 000,000,CYN6
                !byte 000,000,000
                !byte 000,000,000
                !byte 000,000,000
                !byte 000,SCCO,CYN6
                !byte 000,SCCO,CYN6
                !byte PNK,SCCO,CYN6
                !byte SCCO,SCCO,CYN6
                !byte SCCO,SCCO,CYN6
                !byte 000,SCCO,SCCO

;********************************************************************
;'PHOENIXc COPYRIGHT 1980'
;' AMSTAR ELECTRONICS CORP.'
;'  C16 PORT BY SORBAS 2021'
introBottomText
                !byte 000,000,016
                !byte 000,001,008
                !byte 003,013,015
                !byte 033,019,005
                !byte 038,020,014
                !byte 000,001,009
                !byte 016,018,024
                !byte 015,000,029
                !byte 018,005,000
                !byte 020,012,003
                !byte 000,005,015
                !byte 002,003,016
                !byte 025,020,025
                !byte 000,018,018
                !byte 019,015,009
                !byte 015,014,007
                !byte 018,009,008
                !byte 002,003,020
                !byte 001,019,000
                !byte 019,000,033
                !byte 000,003,041
                !byte 034,015,040
                !byte 032,018,032
                !byte 034,016,000
                !byte 033,131,000

introBottomTextColor
                !byte 000,000,CYN6
                !byte 000,CYN6,CYN6
                !byte CYN6,CYN6,CYN6
                !byte CYN6,CYN6,CYN6
                !byte CYN6,CYN6,CYN6
                !byte 000,CYN6,CYN6
                !byte CYN6,CYN6,CYN6
                !byte CYN6,000,PNK
                !byte CYN6,CYN6,000
                !byte CYN6,CYN6,CYN6
                !byte 000,CYN6,CYN6
                !byte CYN6,CYN6,CYN6
                !byte CYN6,CYN6,CYN6
                !byte 000,CYN6,CYN6
                !byte CYN6,CYN6,CYN6
                !byte CYN6,CYN6,CYN6
                !byte CYN6,CYN6,CYN6
                !byte CYN6,CYN6,CYN6
                !byte CYN6,CYN6,000
                !byte CYN6,000,SCCO
                !byte 000,CYN6,SCCO
                !byte SCCO,CYN6,SCCO
                !byte SCCO,CYN6,SCCO
                !byte SCCO,CYN6,000
                !byte SCCO,WHT,000

;Jump table for the over all game flow
gameFlowJumpTableLow
                !byte <TreatFlowStateTitle
                !byte <TreatFlowStateSlowPrint
                !byte <TreatFlowStateAnimation
                !byte <TreatFlowStatePushButton
                !byte <TreatFlowStateInit
                !byte <TreatFlowStateMenu
                !byte <TreatFlowStateCurtainUp
                !byte <TreatFlowStateFadeIn
                !byte <TreatFlowStateAlive
                !byte <TreatFlowStateDying
                !byte <TreatFlowStateGameOver
                !byte <TreatFlowStateTheEnd

gameFlowJumpTableHigh
                !byte >TreatFlowStateTitle
                !byte >TreatFlowStateSlowPrint
                !byte >TreatFlowStateAnimation
                !byte >TreatFlowStatePushButton
                !byte >TreatFlowStateInit
                !byte >TreatFlowStateMenu
                !byte >TreatFlowStateCurtainUp
                !byte >TreatFlowStateFadeIn
                !byte >TreatFlowStateAlive
                !byte >TreatFlowStateDying
                !byte >TreatFlowStateGameOver
                !byte >TreatFlowStateTheEnd

overAllGameWaveTable
                !byte FLOW_WAVE_1
                !byte FLOW_WAVE_1
                !byte FLOW_WAVE_2
                !byte FLOW_WAVE_2
                !byte FLOW_WAVE_3
                !byte FLOW_WAVE_1
                !byte FLOW_WAVE_1
                !byte FLOW_WAVE_2
                !byte FLOW_WAVE_2
                !byte FLOW_WAVE_3

overAllGameFormTable
                !byte FLOW_ALIEN_FORM_A
                !byte FLOW_ALIEN_FORM_B
                !byte FLOW_BIRD_FORM_A
                !byte FLOW_BIRD_FORM_B
                !byte FLOW_ALIEN_FORM_E
                !byte FLOW_ALIEN_FORM_C
                !byte FLOW_ALIEN_FORM_D
                !byte FLOW_BIRD_FORM_A
                !byte FLOW_BIRD_FORM_B
                !byte FLOW_ALIEN_FORM_F

overAllGameColorSetTable
                !byte COLOR_SET1
                !byte COLOR_SET2
                !byte COLOR_SET1
                !byte COLOR_SET2
                !byte COLOR_SET2
                !byte COLOR_SET1
                !byte COLOR_SET2
                !byte COLOR_SET1
                !byte COLOR_SET2
                !byte COLOR_SET2

overAllGameChrSetTable
                !byte CHAR_SETB_BASE
                !byte CHAR_SETB_BASE
                !byte CHAR_SETC_BASE
                !byte CHAR_SETC_BASE
                !byte CHAR_SETB_BASE
                !byte CHAR_SETB_BASE
                !byte CHAR_SETB_BASE
                !byte CHAR_SETC_BASE
                !byte CHAR_SETC_BASE
                !byte CHAR_SETB_BASE

;********************************************************************
; FLOW MODULE END                                                   *
;********************************************************************