;********************************************************************
; GENERAL ROUTINES                                                  *
;********************************************************************
;********************************************************************
; Subroutine:   VideoRegSetup
;********************************************************************
VideoRegSetup
                ;set colors to black
                lda #BLK
                sta TEDREG19            ;Border
                sta TEDREG15            ;Background 0
                sta TEDREG16            ;Background 1
                sta TEDREG17            ;Background 2
                sta TEDREG18            ;Background 3
                ;enable 256 user defined characters
                lda TEDREG07
                ora #RVS_VIDEO_SW
                sta TEDREG07
                ;read char set from RAM
                lda TEDREG12
                and #READ_FROM_RAM
                sta TEDREG12
                rts

;********************************************************************
; Subroutine:   SetCharSet
;********************************************************************
SetCharSet
                ;set the location of custom characters
                lda TEDREG13
                and #%00000011          ;mask out bits for: SClock,ST
                ora CHRSETBASE          ;set the character memory base
                sta TEDREG13
                rts

;********************************************************************
; Subroutine:   InitZeropage
;********************************************************************
InitZeropage
                lda #$00
                ldx #ZP_LAST
.initzeropage_loop
                sta $00,x
                dex
                cpx #$01                ;leave $00 and $01
                bne .initzeropage_loop
                rts

;********************************************************************
; Subroutine:   WaitForIrq
; Input:        WAITIRQ
;********************************************************************
WaitForIrq
                ;wait util IRQ is finished and WAITIRQ becomes 1
.waitforirq_loop
                lda WAITIRQ
                beq .waitforirq_loop
                lda #$00
                sta WAITIRQ
                rts

;********************************************************************
; Subroutine:   Wait500
;********************************************************************
Wait500
                ldy #$10
                ldx #$FF
.wait500_loop
                dex
                nop
                bne .wait500_loop
                dey
                bne .wait500_loop
                rts

;********************************************************************
; Subroutine:   LoadTitleColorSet
; Input:        TITLECOLORSET:     COLOR_SET1 / COLOR_SET2
;********************************************************************
LoadTitleColorSet
                lda TITLECOLORSET
                cmp #COLOR_SET1
                bne .loadtitlecolorset_toggle
                lda #COLOR_SET2
                sta TITLECOLORSET
;set 1 **************************************************************
                ;for big game title
                lda #$FA
                sta MEMORYRANGE
                lda #<titleColorSet1
                sta SOURCEADDRESS
                lda #>titleColorSet1
                sta SOURCEADDRESS+1
                lda #<gameIntroPage2L1Color
                sta TARGETADDRESS
                lda #>gameIntroPage2L1Color
                sta TARGETADDRESS+1
                jsr CopyMemory
                ;for bird animation
                lda #$FC
                sta MEMORYRANGE
                lda #<birdColorSet1
                sta SOURCEADDRESS
                lda #>birdColorSet1
                sta SOURCEADDRESS+1
                lda #<bird13Color
                sta TARGETADDRESS
                lda #>bird13Color
                sta TARGETADDRESS+1
                jsr CopyMemory
                ;for the4Shapes
                lda #$3C
                sta MEMORYRANGE
                lda #<the4ShapesColorSet1
                sta SOURCEADDRESS
                lda #>the4ShapesColorSet1
                sta SOURCEADDRESS+1
                lda #<the4ShapesColor
                sta TARGETADDRESS
                lda #>the4ShapesColor
                sta TARGETADDRESS+1
                jsr CopyMemory
                ;lives symbol
                lda #$6B
                sta topTextColor+6
                sta topTextColor+63
                sta introBottomTextColor+23
                rts
.loadtitlecolorset_toggle
                lda #COLOR_SET1
                sta TITLECOLORSET
;set 2 **************************************************************
                ;colors for big game title
                lda #$FA
                sta MEMORYRANGE
                lda #<titleColorSet2
                sta SOURCEADDRESS
                lda #>titleColorSet2
                sta SOURCEADDRESS+1
                lda #<gameIntroPage2L1Color
                sta TARGETADDRESS
                lda #>gameIntroPage2L1Color
                sta TARGETADDRESS+1
                jsr CopyMemory
                ;for bird animation
                lda #$FC
                sta MEMORYRANGE
                lda #<birdColorSet2
                sta SOURCEADDRESS
                lda #>birdColorSet2
                sta SOURCEADDRESS+1
                lda #<bird13Color
                sta TARGETADDRESS
                lda #>bird13Color
                sta TARGETADDRESS+1
                jsr CopyMemory
                ;for the4Shapes
                lda #$3C
                sta MEMORYRANGE
                lda #<the4ShapesColorSet2
                sta SOURCEADDRESS
                lda #>the4ShapesColorSet2
                sta SOURCEADDRESS+1
                lda #<the4ShapesColor
                sta TARGETADDRESS
                lda #>the4ShapesColor
                sta TARGETADDRESS+1
                jsr CopyMemory
                ;lives symbol
                lda #$5F
                sta topTextColor+6
                sta topTextColor+63
                sta introBottomTextColor+23
                rts

;********************************************************************
; Subroutine:   LoadAlienColorSet
; Input:        ALIENCOLORSET:     COLOR_SET1 / COLOR_SET2
;********************************************************************
LoadAlienColorSet
                lda ALIENCOLORSET
                cmp #COLOR_SET1
                bne .loadaliencolorset_toggle
                lda #COLOR_SET2
                sta ALIENCOLORSET
;set 1 **************************************************************
                ;color for aliens
                lda #$5C                ;32+8+24+4+24
                sta MEMORYRANGE
                lda #<alienShapeColorSet1
                sta SOURCEADDRESS
                lda #>alienShapeColorSet1
                sta SOURCEADDRESS+1
                lda #<alienShape01Color
                sta TARGETADDRESS
                lda #>alienShape01Color
                sta TARGETADDRESS+1
                jsr CopyMemory
                ;lives symbol
                lda #$6B
                sta topTextColor+6
                sta topTextColor+63
                sta introBottomTextColor+23
                rts
.loadaliencolorset_toggle
                lda #COLOR_SET1
                sta ALIENCOLORSET
;set 2 **************************************************************
                ;color for aliens
                lda #$5C
                sta MEMORYRANGE
                lda #<alienShapeColorSet2
                sta SOURCEADDRESS
                lda #>alienShapeColorSet2
                sta SOURCEADDRESS+1
                lda #<alienShape01Color
                sta TARGETADDRESS
                lda #>alienShape01Color
                sta TARGETADDRESS+1
                jsr CopyMemory
                ;lives symbol
                lda #$5F
                sta topTextColor+6
                sta topTextColor+63
                sta introBottomTextColor+23
.loadaliencolorset_exit
                rts

;********************************************************************
; Subroutine:   CopyMemory
; Input:        SOURCEADDRESS:      Pointer: source address
;               TARGETADDRESS:      Pointer: target address
;               MEMORYRANGE:        amount of bytes to be copied
;********************************************************************
CopyMemory
                ldy #$00
.copymemory_loop
                lda (SOURCEADDRESS),Y
                sta (TARGETADDRESS),Y
                iny
                cpy MEMORYRANGE
                bne .copymemory_loop
                rts

;********************************************************************
; Subroutine:   CopyNotZeroData
; Input:        SOURCEADDRESS:      Pointer: source address
;               TARGETADDRESS:      Pointer: target address
;               MEMORYRANGE:        amount of bytes to be copied
;********************************************************************
CopyNotZeroData
                ldy #$00
.copynotzerodata_loop
                lda (SOURCEADDRESS),Y
                beq .copynotzerodata_skip
                sta (TARGETADDRESS),Y
.copynotzerodata_skip
                iny
                cpy MEMORYRANGE
                bne .copynotzerodata_loop
                rts

;********************************************************************
; Subroutine:   StoreSeed
; Seed, can be anything except 0.
; Seed will come from the current horizontal scan pos.,
; at first joystick fire button click.
;********************************************************************
StoreSeed
                lda NEWRANDOMSEED
                sta PSEUDORANDOML
                lda #$00
                sta PSEUDORANDOMH
                rts

;********************************************************************
; Subroutine:   GetRandomValue
; Original idea was '16-bit "798" Xorshift'
; A fast pseudorandom generator algorithm.
; It has a period of 65535 and passes reasonable tests for randomness.
; You can get 8-bit random numbers in A or 16-bit numbers
; from the zero page addresses. Leaves X/Y unchanged.
;********************************************************************
GetRandomValue
                lda PSEUDORANDOMH
                lsr
                lda PSEUDORANDOML
                ror
                eor PSEUDORANDOMH
                sta PSEUDORANDOMH   ;high part of x ^= x << 7 done
                ror                 ;A has now x >> 9 and high bit comes from low byte
                eor PSEUDORANDOML
                sta PSEUDORANDOML   ;x ^= x >> 9 and the low part of x ^= x << 7 done
                eor PSEUDORANDOMH
                sta PSEUDORANDOMH   ;x ^= x << 8 done
                rts

;********************************************************************
; Subroutine:   GetRandomValueWithInterval
; Input:        A-reg:              max value - 1
; Output:       X-reg:              random value between 0 and A-reg-1
;********************************************************************
GetRandomValueWithInterval
                pha
                jsr GetRandomValue  ;get 8-bit random number in A
                ;trimm down to a value between 0 and A-reg-1.
                sta MULTIPLICAND
                pla
                sta MULTIPLIER
                lda #$00
                sta PRODUCTL
                sta PRODUCTH
                jsr Multiplication
                ldx PRODUCTH
                rts

;********************************************************************
; Subroutine:   Multiplication
; 8-bit by 8-bit unsigned multiplication.
; This subroutine multiplies an 8-bit unsigned multiplicand:
; MULTIPLICAND by an 8-bit unsigned multiplier: MULTIPLIER,
; and returns the 16-bit unsigned product: PRODUCTL (low byte)
; and PRODUCTH (high byte).
;********************************************************************
Multiplication
                lda #$00            ;clear high byte of product
                ldx #$08            ;multiplier bit count=8
.multiplication_nxtbt
                lsr MULTIPLIER      ;get next multiplier bit
                                    ;multiplier=1 ?
                bcc .multiplication_align
                clc                 ;yes. add multiplicand
                adc MULTIPLICAND
.multiplication_align
                lsr                 ;shift product right
                ror PRODUCTL
                dex                 ;decrement bit count
                                    ;loop until 8 bits are done
                bne .multiplication_nxtbt
                sta PRODUCTH        ;store product high byte
                rts

;********************************************************************
; GENERAL ROUTINES END                                              *
;********************************************************************