;********************************************************************
; SHIELD MODULE                                                     *
;********************************************************************
;********************************************************************
; Subroutine:   ShieldUpdate
;********************************************************************
ShieldUpdate
                lda PLAYERACTIVE
                cmp #FALSE
                beq .shieldupdate_exit
                ldy #TIMER_5
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .shieldupdate_forbidden
                lda #FALSE
                sta shieldForbidden
                ;reset timer
                ldy #TIMER_5
                jsr ResetTimer
.shieldupdate_forbidden
                jsr ShieldInputUpdate
                jsr ShieldStatusUpdate
                jsr ShieldDrawUpdate
                jsr ShieldFrameUpdate
.shieldupdate_exit
                rts

;********************************************************************
; Subroutine:   ShieldInputUpdate
;********************************************************************
ShieldInputUpdate
                lda GAMEPORTTHISFRAME
                and #UP_MASK
                beq .shieldinputupdate_up
                lda GAMEPORTTHISFRAME
                and #DOWN_MASK
                bne .shieldinputupdate_noupdown
.shieldinputupdate_up
                ;up or down *****************************************
                lda shieldForbidden
                cmp #FALSE
                bne .shieldinputupdate_noupdown
                lda #TRUE
                sta UPKEY
                rts
.shieldinputupdate_noupdown
                lda #FALSE
                sta UPKEY
.shieldinputupdate_done
                rts

;********************************************************************
; Subroutine:   ShieldStatusUpdate
;********************************************************************
ShieldStatusUpdate
                lda UPKEY
                cmp #TRUE
                bne .shieldstatusupdate_noupkey
                ldy #TIMER_4
                lda timerState,Y
                cmp #TIMER_STATE_INACTIVE
                bne .shieldstatusupdate_timeractivethreshold
                ;trigger sound shield
                jsr TriggerSoundShield
                ;start timer
                ldy #TIMER_4
                jsr StartTimer
                lda #TRUE
                sta shieldAction
                sta drawShield
                lda #FALSE
                sta deleteShield
                sta newShieldAllowed
                ;get position
                ldy playerX
                dey                 ;align ship position with shield position
                sty SHIELDXPOS
                ldy playerY
                dey
                sty SHIELDYPOS
                rts
.shieldstatusupdate_timeractivethreshold
                ldy #TIMER_4
                lda timerState,Y
                cmp #TIMER_STATE_ACTIVE
                bne .shieldstatusupdate_timerthreshold
                lda #TRUE
                sta shieldAction
                lda #FALSE
                sta drawShield
                sta deleteShield
                sta newShieldAllowed
                rts
.shieldstatusupdate_timerthreshold
                ;reset timer
                ldy #TIMER_4
                jsr ResetTimer
                lda #TRUE
                sta shieldAction
                sta deleteShield
                sta newShieldAllowed
                lda #FALSE
                sta drawShield
                rts
.shieldstatusupdate_noupkey
                ldy #TIMER_4
                lda timerState,Y
                cmp #TIMER_STATE_INACTIVE
                bne .shieldstatusupdate_timeractivethreshold
.shieldstatusupdate_forbidden
                lda #TRUE
                sta newShieldAllowed
                lda #FALSE
                sta shieldAction
                sta drawShield
                sta deleteShield
.shieldstatusupdate_exit
                rts

;********************************************************************
; Subroutine:   ShieldDrawUpdate
;********************************************************************
ShieldDrawUpdate
                lda shieldAction
                cmp #TRUE
                bne .shielddrawupdate_exit
                lda drawShield
                cmp #TRUE
                bne .shielddrawupdate_nextcheck
                ;start shield animation timer
                ldy #TIMER_3
                jsr StartTimer
                ;draw the shield
                jsr DrawShield
                lda #FALSE
                sta newShieldAllowed
                ;ship movement and animation locked
                sta playerMoveAllowed
                sta playerFrameChangeAllowed
                ;player is now invulnerable
                lda #TRUE
                sta PLAYERINVULNERABLE
                ;reset player frame
                lda #$00
                sta PLAYERFRAME
                rts
.shielddrawupdate_nextcheck
                lda deleteShield
                cmp #TRUE
                bne .shielddrawupdate_exit
                ;stop shield animation timer
                ldy #TIMER_3
                jsr ResetTimer
                ;delete the shield
                jsr DelShield
                lda #FALSE
                sta shieldAction
                lda #TRUE
                sta newShieldAllowed
                ;ship movement and animation unlocked
                sta playerMoveAllowed
                sta playerFrameChangeAllowed
                ;player is no longer invulnerable
                lda #FALSE
                sta PLAYERINVULNERABLE
                ;new shield forbidden until timer threshold
                lda #TRUE
                sta shieldForbidden
                ldy #TIMER_5
                jsr StartTimer
.shielddrawupdate_exit
                rts

;********************************************************************
; Subroutine:   ShieldFrameUpdate
;********************************************************************
ShieldFrameUpdate
                ldy #TIMER_4
                lda timerState,Y
                cmp #TIMER_STATE_ACTIVE
                bne .shieldframeupdate_notask
                jsr RotateShieldFrame
.shieldframeupdate_notask
                rts

;********************************************************************
; Subroutine:   RotateShieldFrame
;********************************************************************
RotateShieldFrame
                ldy #TIMER_3
                lda timerState,Y
                cmp #TIMER_STATE_THRESHOLD
                bne .rotateshieldframe_exit
                jsr RestartTimer
                ;do task ********************************************
                inc SHIELDFRAME
                lda SHIELDFRAME
                cmp #$06
                bne .rotateshieldframe_cont
                lda #$00
                sta SHIELDFRAME
.rotateshieldframe_cont
                jsr ReplaceCharacters
.rotateshieldframe_exit
                jsr DrawShield
                rts

;********************************************************************
; Subroutine:   ReplaceCharacters
;********************************************************************
ReplaceCharacters
                tax
                lda shieldAddressTableLow,X
                sta SOURCEADDRESS
                lda shieldAddressTableHigh,X
                sta SOURCEADDRESS+1
                lda #$60
                sta MEMORYRANGE
                lda CHRSETBASE
                cmp #CHAR_SETB_BASE
                bne .replacecharacters_setc
                lda #<shieldChrBRam
                sta TARGETADDRESS
                lda #>shieldChrBRam
                sta TARGETADDRESS+1
                jmp .replacecharacters_copy
.replacecharacters_setc
                lda #<shieldChrCRam
                sta TARGETADDRESS
                lda #>shieldChrCRam
                sta TARGETADDRESS+1
.replacecharacters_copy
                jsr CopyMemory
                rts

;********************************************************************
; Subroutine:   DrawShield
;********************************************************************
DrawShield
                lda SHIELDXPOS
                sta CURRENTSHAPEXPOS
                lda SHIELDYPOS
                sta CURRENTSHAPEYPOS
                ldx SHIELDFRAME
                lda #<shield
                sta STRINGL
                lda #>shield
                sta STRINGH
                lda shieldColorFrameLow,X
                sta TEXTCOLORL
                lda shieldColorFrameHigh,X
                sta TEXTCOLORH
                lda #$04
                sta BLOCKWIDTH
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH
                sta MAXSCREENX
                jsr DrawShape
                rts

;********************************************************************
; Subroutine:   DelShield
;********************************************************************
DelShield
                lda SHIELDXPOS
                sta CURRENTSHAPEXPOS
                lda SHIELDYPOS
                sta CURRENTSHAPEYPOS
                lda #<clrShield
                sta STRINGL
                lda #>clrShield
                sta STRINGH
                lda #<clrShieldColor
                sta TEXTCOLORL
                lda #>clrShieldColor
                sta TEXTCOLORH
                lda #$04
                sta BLOCKWIDTH
                sta BLOCKHEIGHT
                lda #SCREEN_WIDTH
                sta MAXSCREENX
                jsr DrawShape
                rts

;********************************************************************
;game shield data ***************************************************
;********************************************************************

shieldAction
                !byte FALSE

newShieldAllowed
                !byte TRUE

drawShield
                !byte FALSE

deleteShield
                !byte FALSE

shieldForbidden
                !byte FALSE

shield
                !byte 076,074,072,068
                !byte 077,058,056,069
                !byte 078,059,057,070
                !byte 079,075,073,071

shield1Color
                !byte BLU,BLU,BLU,BLU
                !byte BLU,PNK3,PNK3,BLU
                !byte BLU,PNK5,PNK5,BLU
                !byte BLU,BLU,BLU,BLU

shield2Color
                !byte CYN6,CYN6,CYN6,CYN6
                !byte CYN6,PNK3,PNK3,CYN6
                !byte CYN6,PNK5,PNK5,CYN6
                !byte CYN6,CYN6,CYN6,CYN6

shield3Color
                !byte PUR,PUR,PUR,PUR
                !byte PUR,PNK5,PNK5,PUR
                !byte PUR,PNK3,PNK3,PUR
                !byte PUR,PUR,PUR,PUR

shield4Color
                !byte LBLU,LBLU,LBLU,LBLU
                !byte LBLU,PNK5,PNK5,LBLU
                !byte LBLU,PNK3,PNK3,LBLU
                !byte LBLU,LBLU,LBLU,LBLU

;shield color frame table
shieldColorFrameLow
                !byte <shield1Color
                !byte <shield2Color
                !byte <shield3Color
                !byte <shield4Color
                !byte <shield3Color
                !byte <shield2Color

shieldColorFrameHigh
                !byte >shield1Color
                !byte >shield2Color
                !byte >shield3Color
                !byte >shield4Color
                !byte >shield3Color
                !byte >shield2Color

;delete shield
clrShield
                !byte CHR_SPACE,CHR_SPACE,CHR_SPACE,CHR_SPACE
                !byte CHR_SPACE,CHR_SPACE,CHR_SPACE,CHR_SPACE
                !byte CHR_SPACE,CHR_SPACE,CHR_SPACE,CHR_SPACE
                !byte CHR_SPACE,CHR_SPACE,CHR_SPACE,CHR_SPACE

clrShieldColor
                !byte WHT,WHT,WHT,WHT
                !byte WHT,WHT,WHT,WHT
                !byte WHT,WHT,WHT,WHT
                !byte WHT,WHT,WHT,WHT

;shield character data will be copied to shieldChrBRam or shieldChrCRam
shield1ChrData                              ;frame 4: exterior shield
    !byte   $00,$00,$C0,$20,$10,$08,$04,$04
    !byte   $02,$02,$02,$01,$01,$01,$01,$01
    !byte   $01,$01,$01,$01,$01,$02,$02,$02
    !byte   $04,$04,$08,$10,$20,$C0,$00,$00
    !byte   $F8,$07,$00,$00,$00,$00,$00,$00
    !byte   $00,$00,$00,$00,$00,$00,$07,$F8
    !byte   $1F,$E0,$00,$00,$00,$00,$00,$00
    !byte   $00,$00,$00,$00,$00,$00,$E0,$1F
    !byte   $00,$00,$03,$04,$08,$10,$20,$20
    !byte   $40,$40,$40,$80,$80,$80,$80,$80
    !byte   $80,$80,$80,$80,$80,$40,$40,$40
    !byte   $20,$20,$10,$08,$04,$03,$00,$00

shield2ChrData                              ;frame 3: middle2 shield
    !byte   $00,$00,$00,$C0,$60,$30,$18,$08
    !byte   $0C,$04,$04,$06,$02,$02,$02,$02
    !byte   $02,$02,$02,$02,$06,$04,$04,$0C
    !byte   $08,$18,$30,$60,$C0,$00,$00,$00
    !byte   $00,$F8,$0F,$01,$00,$00,$00,$00
    !byte   $00,$00,$00,$00,$01,$0F,$F8,$00
    !byte   $00,$1F,$F0,$80,$00,$00,$00,$00
    !byte   $00,$00,$00,$00,$80,$F0,$1F,$00
    !byte   $00,$00,$00,$03,$06,$0C,$18,$10
    !byte   $30,$20,$20,$60,$40,$40,$40,$40
    !byte   $40,$40,$40,$40,$60,$20,$20,$30
    !byte   $10,$18,$0C,$06,$03,$00,$00,$00

shield3ChrData                              ;frame 2: middle1 shield
    !byte   $00,$00,$00,$00,$80,$C0,$E0,$70
    !byte   $30,$18,$18,$18,$0C,$0C,$0C,$0C
    !byte   $0C,$0C,$0C,$0C,$18,$18,$18,$30
    !byte   $70,$E0,$C0,$80,$00,$00,$00,$00
    !byte   $00,$00,$F0,$FE,$0F,$01,$00,$00
    !byte   $00,$00,$01,$0F,$FE,$F0,$00,$00
    !byte   $00,$00,$0F,$7F,$F0,$80,$00,$00
    !byte   $00,$00,$80,$F0,$7F,$0F,$00,$00
    !byte   $00,$00,$00,$00,$01,$03,$07,$0E
    !byte   $0C,$18,$18,$18,$30,$30,$30,$30
    !byte   $30,$30,$30,$30,$18,$18,$18,$0C
    !byte   $0E,$07,$03,$01,$00,$00,$00,$00

shield4ChrData                              ;frame 1: inner shield
    !byte   $00,$00,$00,$00,$00,$00,$00,$80
    !byte   $C0,$E0,$60,$60,$70,$30,$30,$30
    !byte   $30,$30,$30,$70,$60,$60,$E0,$C0
    !byte   $80,$00,$00,$00,$00,$00,$00,$00
    !byte   $00,$00,$00,$00,$F0,$FE,$1F,$03
    !byte   $03,$1F,$FE,$F0,$00,$00,$00,$00
    !byte   $00,$00,$00,$00,$0F,$7F,$F8,$C0
    !byte   $C0,$F8,$7F,$0F,$00,$00,$00,$00
    !byte   $00,$00,$00,$00,$00,$00,$00,$01
    !byte   $03,$07,$06,$06,$0E,$0C,$0C,$0C
    !byte   $0C,$0C,$0C,$0E,$06,$06,$07,$03
    !byte   $01,$00,$00,$00,$00,$00,$00,$00

shieldAddressTableLow
                !byte <shield4ChrData
                !byte <shield3ChrData
                !byte <shield2ChrData
                !byte <shield1ChrData
                !byte <shield2ChrData
                !byte <shield3ChrData

shieldAddressTableHigh
                !byte >shield4ChrData
                !byte >shield3ChrData
                !byte >shield2ChrData
                !byte >shield1ChrData
                !byte >shield2ChrData
                !byte >shield3ChrData

;********************************************************************
; SHIELD MODULE END                                                 *
;********************************************************************