;********************************************************************
; SOUND MODULE                                                      *
;********************************************************************
;********************************************************************
; Subroutine:   AudioRegSetup
;********************************************************************
AudioRegSetup
                ;Set volume to 0
                lda #$00
                sta TEDREG11
                rts

;********************************************************************
; Subroutine:   SoundUpdate
;               Called during raster interrupt
;********************************************************************
SoundUpdate
                ldx #VOICE_1
.soundupdate_loop
                lda soundVoiceActive,X
                bne .soundupdate_active
                jmp .soundupdate_done
.soundupdate_active
                lda soundDelay,X
                beq .soundupdate_process
                dec soundDelay,X
                jmp .soundupdate_done
.soundupdate_process
                lda soundCmdPtrLow,X
                sta SOUNDCMDL
                lda soundCmdPtrHigh,X
                sta SOUNDCMDH
                ldy soundCmdBufferIndex,X
                lda (SOUNDCMDL),Y
.soundupdate_processloop
                ;process the command buffer
                jsr ProcessVolume
                jsr ProcessVoice
                jsr ProcessFrequencyHigh
                jsr ProcessFrequencyLow
                ;command delay
                cmp #CMD_DELAY
                bne .soundupdate_delayskip
                inc soundCmdBufferIndex,X    ;move to next byte
                ldy soundCmdBufferIndex,X
                lda (SOUNDCMDL),Y
                sta soundDelay,X
                inc soundCmdBufferIndex,X    ;move to next byte
                ldy soundCmdBufferIndex,X
                lda (SOUNDCMDL),Y
                jmp .soundupdate_done
.soundupdate_delayskip
                ;command end
                cmp #CMD_END
                bne .soundupdate_endskip
                lda #FALSE
                sta soundVoiceActive,X
                sta soundCmdBufferIndex,X
                jmp .soundupdate_done
.soundupdate_endskip
                jmp .soundupdate_processloop
.soundupdate_done
                inx
                cpx #VOICES_MAX
                beq .soundupdate_finished
                jmp .soundupdate_loop
.soundupdate_finished
                rts

;********************************************************************
; Subroutine:   ProcessVolume
;********************************************************************
ProcessVolume
                cmp #CMD_VOLUME
                bne .processvolume_skip
                inc soundCmdBufferIndex,X    ;move to next byte
                ldy soundCmdBufferIndex,X
                lda (SOUNDCMDL),Y
                sta VOLUME
                jsr SetVolume
                inc soundCmdBufferIndex,X    ;move to next byte
                ldy soundCmdBufferIndex,X
                lda (SOUNDCMDL),Y
.processvolume_skip
                rts

;********************************************************************
; Subroutine:   ProcessVoice
;********************************************************************
ProcessVoice
                cmp #CMD_VOICE
                bne .processvoice_skip
                inc soundCmdBufferIndex,X    ;move to next byte
                ldy soundCmdBufferIndex,X
                lda (SOUNDCMDL),Y
                cpx #VOICE_1
                bne .processvoice_voice2
                cmp #SQUARE1_START
                bne .processvoice_next1
                jsr ActivateVoice1
                jmp .processvoice_done
.processvoice_next1
                cmp #SQUARE1_END
                bne .processvoice_done
                jsr DeactivateVoice1
                jmp .processvoice_done
.processvoice_voice2
                cpx #VOICE_2
                bne .processvoice_voice3
                cmp #SQUARE2_START
                bne .processvoice_next2
                jsr ActivateVoice2
                jmp .processvoice_done
.processvoice_next2
                cmp #SQUARE2_END
                bne .processvoice_done
                jsr DeactivateVoice2
                jmp .processvoice_done
.processvoice_voice3
                cmp #NOISE_START
                bne .processvoice_next3
                jsr ActivateNoise
                jmp .processvoice_done
.processvoice_next3
                cmp #NOISE_END
                bne .processvoice_done
                jsr DeactivateNoise
.processvoice_done
                inc soundCmdBufferIndex,X    ;move to next byte
                ldy soundCmdBufferIndex,X
                lda (SOUNDCMDL),Y
.processvoice_skip
                rts

;********************************************************************
; Subroutine:   ProcessFrequencyHigh
;********************************************************************
ProcessFrequencyHigh
                cmp #CMD_FREQUENCYHIGH
                bne .processfrequencyhigh_skip
                inc soundCmdBufferIndex,X    ;move to next byte
                ldy soundCmdBufferIndex,X
                lda (SOUNDCMDL),Y
                cpx #VOICE_1
                bne .processfrequencyhigh_voice2
                sta FREQ1HIGH
                jsr SetFreq1high
                jmp .processfrequencyhigh_done
.processfrequencyhigh_voice2
                cpx #VOICE_2
                bne .processfrequencyhigh_voice3
                sta FREQ2HIGH
                jsr SetFreq2high
                jmp .processfrequencyhigh_done
.processfrequencyhigh_voice3
                sta FREQ2HIGH
                jsr SetFreq2high
.processfrequencyhigh_done
                inc soundCmdBufferIndex,X    ;move to next byte
                ldy soundCmdBufferIndex,X
                lda (SOUNDCMDL),Y
.processfrequencyhigh_skip
                rts

;********************************************************************
; Subroutine:   ProcessFrequencyLow
;********************************************************************
ProcessFrequencyLow
                cmp #CMD_FREQUENCYLOW
                bne .processfrequencylow_skip
                inc soundCmdBufferIndex,X    ;move to next byte
                ldy soundCmdBufferIndex,X
                lda (SOUNDCMDL),Y
                cpx #VOICE_1
                bne .processfrequencylow_voice2
                sta FREQ1LOW
                jsr SetFreq1low
                jmp .processfrequencylow_done
.processfrequencylow_voice2
                cpx #VOICE_2
                bne .processfrequencylow_voice3
                sta FREQ2LOW
                jsr SetFreq2low
                jmp .processfrequencylow_done
.processfrequencylow_voice3
                sta FREQ2LOW
                jsr SetFreq2low
.processfrequencylow_done
                inc soundCmdBufferIndex,X    ;move to next byte
                ldy soundCmdBufferIndex,X
                lda (SOUNDCMDL),Y
.processfrequencylow_skip
                rts

;********************************************************************
; Subroutine:   SetVolume
;********************************************************************
SetVolume
                lda TEDREG11
                and #VOLUME_MASK
                ora VOLUME
                sta TEDREG11
                rts

;********************************************************************
; Subroutine:   SetFreq1low
;********************************************************************
SetFreq1low
                lda FREQ1LOW
                sta TEDREG0E
                rts

;********************************************************************
; Subroutine:   SetFreq1high
;********************************************************************
SetFreq1high
                lda TEDREG12
                and #FREQHIGH_MASK
                ora FREQ1HIGH
                sta TEDREG12
                rts

;********************************************************************
; Subroutine:   SetFreq2low
;********************************************************************
SetFreq2low
                lda FREQ2LOW
                sta TEDREG0F
                rts

;********************************************************************
; Subroutine:   SetFreq2high
;********************************************************************
SetFreq2high
                lda TEDREG10
                and #FREQHIGH_MASK
                ora FREQ2HIGH
                sta TEDREG10
                rts

;********************************************************************
; Subroutine:   ActivateVoice1
;********************************************************************
ActivateVoice1
                lda TEDREG11
                ora #VOICE1_ON
                sta TEDREG11
                rts

;********************************************************************
; Subroutine:   ActivateVoice2
;********************************************************************
ActivateVoice2
                lda TEDREG11
                ora #VOICE2_ON
                sta TEDREG11
                rts

;********************************************************************
; Subroutine:   ActivateNoise
;********************************************************************
ActivateNoise
                lda TEDREG11
                ora #NOISE_ON
                sta TEDREG11
                rts

;********************************************************************
; Subroutine:   DeactivateVoice1
;********************************************************************
DeactivateVoice1
                lda TEDREG11
                and #VOICE1_MASK
                sta TEDREG11
                rts

;********************************************************************
; Subroutine:   DeactivateVoice2
;********************************************************************
DeactivateVoice2
                lda TEDREG11
                and #VOICE2_MASK
                sta TEDREG11
                rts

;********************************************************************
; Subroutine:   DeactivateNoise
;********************************************************************
DeactivateNoise
                lda TEDREG11
                and #NOISE_MASK
                sta TEDREG11
                rts

;********************************************************************
;sound control data *************************************************
;********************************************************************
soundVoiceActive
                !byte 0, 0, 0

soundCmdPtrHigh
                !byte 0, 0, 0

soundCmdPtrLow
                !byte 0, 0, 0

soundCmdBufferIndex
                !byte 0, 0, 0

soundDelay
                !byte 0, 0, 0

;********************************************************************
; Subroutine:   TriggerSoundPlayerFiring
;********************************************************************
TriggerSoundPlayerFiring
                ldx #VOICE_2
                lda #>soundPlayerFiring
                sta soundCmdPtrHigh,X
                lda #<soundPlayerFiring
                sta soundCmdPtrLow,X
                lda #TRUE
                sta soundVoiceActive,X
                lda #0
                sta soundCmdBufferIndex,X
                rts

soundPlayerFiring
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE2_START
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $30
                !byte CMD_DELAY, 1
                !byte CMD_VOLUME, $07
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $80
                !byte CMD_DELAY, 1
                !byte CMD_VOLUME, $06
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $D0
                !byte CMD_DELAY, 1
                !byte CMD_VOLUME, $05
                !byte CMD_FREQUENCYHIGH, $01
                !byte CMD_FREQUENCYLOW, $80
                !byte CMD_DELAY, 2
                !byte CMD_VOLUME, $04
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $40
                !byte CMD_DELAY, 3
                !byte CMD_VOICE, SQUARE2_END
                
                !byte CMD_END

;********************************************************************
; Subroutine:   TriggerSoundAlienFiring
;********************************************************************
TriggerSoundAlienFiring
                ldx #VOICE_3
                lda #>soundAlienFiring
                sta soundCmdPtrHigh,X
                lda #<soundAlienFiring
                sta soundCmdPtrLow,X
                lda #TRUE
                sta soundVoiceActive,X
                lda #0
                sta soundCmdBufferIndex,X
                rts

soundAlienFiring
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, NOISE_START
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $81
                !byte CMD_DELAY, 1
                !byte CMD_VOLUME, $07
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $1E
                !byte CMD_DELAY, 1
                !byte CMD_VOLUME, $06
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $02
                !byte CMD_DELAY, 2
                !byte CMD_VOLUME, $05
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $3B
                !byte CMD_DELAY, 3
                !byte CMD_VOLUME, $04
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $04
                !byte CMD_DELAY, 4
                !byte CMD_VOICE, NOISE_END
                
                !byte CMD_END

;********************************************************************
; Subroutine:   TriggerSoundAlienExplosion
;********************************************************************
TriggerSoundAlienExplosion
                ldx #VOICE_1
                lda #>soundAlienExplosion
                sta soundCmdPtrHigh,X
                lda #<soundAlienExplosion
                sta soundCmdPtrLow,X
                lda #TRUE
                sta soundVoiceActive,X
                lda #0
                sta soundCmdBufferIndex,X
                rts

soundAlienExplosion
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $76
                !byte CMD_DELAY, 2
                !byte CMD_VOLUME, $07
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $3B
                !byte CMD_DELAY, 3
                !byte CMD_VOLUME, $06
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $76
                !byte CMD_DELAY, 4
                !byte CMD_VOICE, SQUARE1_END
                
                !byte CMD_END

;********************************************************************
; Subroutine:   TriggerSoundParticleExplosion
;********************************************************************
TriggerSoundParticleExplosion
                ldx #VOICE_3
                lda #>soundParticleExplosion
                sta soundCmdPtrHigh,X
                lda #<soundParticleExplosion
                sta soundCmdPtrLow,X
                lda #TRUE
                sta soundVoiceActive,X
                lda #0
                sta soundCmdBufferIndex,X
                rts

soundParticleExplosion
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, NOISE_START
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $D0
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $CA
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $60
                !byte CMD_DELAY, 2
                !byte CMD_VOLUME, $07
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $42
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $1E
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $E3
                !byte CMD_DELAY, 3
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $AD
                !byte CMD_DELAY, 3
                !byte CMD_VOLUME, $06
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $83
                !byte CMD_DELAY, 3
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $54
                !byte CMD_DELAY, 3
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $3B
                !byte CMD_DELAY, 3
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $04
                !byte CMD_DELAY, 3
                !byte CMD_VOLUME, $05
                !byte CMD_FREQUENCYHIGH, $01
                !byte CMD_FREQUENCYLOW, $7F
                !byte CMD_DELAY, 4
                !byte CMD_FREQUENCYHIGH, $01
                !byte CMD_FREQUENCYLOW, $C5
                !byte CMD_DELAY, 4
                !byte CMD_FREQUENCYHIGH, $01
                !byte CMD_FREQUENCYLOW, $7F
                !byte CMD_DELAY, 4
                !byte CMD_FREQUENCYHIGH, $01
                !byte CMD_FREQUENCYLOW, $59
                !byte CMD_DELAY, 4
                !byte CMD_VOLUME, $04
                !byte CMD_FREQUENCYHIGH, $01
                !byte CMD_FREQUENCYLOW, $06
                !byte CMD_DELAY, 5
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $A9
                !byte CMD_DELAY, 5
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $76
                !byte CMD_DELAY, 5
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $40
                !byte CMD_DELAY, 5
                !byte CMD_VOLUME, $03
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $07
                !byte CMD_DELAY, 5
                !byte CMD_VOICE, NOISE_END
                
                !byte CMD_END

;********************************************************************
; Subroutine:   TriggerSoundGetReady
;********************************************************************
TriggerSoundGetReady
                ldx #VOICE_1
                lda #>soundGetReady
                sta soundCmdPtrHigh,X
                lda #<soundGetReady
                sta soundCmdPtrLow,X
                lda #TRUE
                sta soundVoiceActive,X
                lda #0
                sta soundCmdBufferIndex,X
                rts

soundGetReady
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $01
                !byte CMD_FREQUENCYLOW, $C5
                !byte CMD_DELAY, 6
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 5
                !byte CMD_VOLUME, $07
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $01
                !byte CMD_FREQUENCYLOW, $E5
                !byte CMD_DELAY, 5
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 4
                !byte CMD_VOLUME, $07
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $04
                !byte CMD_DELAY, 4
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 3
                !byte CMD_VOLUME, $06
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $20
                !byte CMD_DELAY, 3
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                !byte CMD_VOLUME, $05
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $3B
                !byte CMD_DELAY, 2
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                !byte CMD_VOLUME, $05
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $54
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                !byte CMD_VOLUME, $04
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $6C
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                !byte CMD_VOLUME, $04
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $83
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_END

;********************************************************************
; Subroutine:   TriggerSoundShield
;********************************************************************
TriggerSoundShield
                ldx #VOICE_2
                lda #>soundShield
                sta soundCmdPtrHigh,X
                lda #<soundShield
                sta soundCmdPtrLow,X
                lda #TRUE
                sta soundVoiceActive,X
                lda #0
                sta soundCmdBufferIndex,X
                rts

soundShield
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE2_START
                !byte CMD_FREQUENCYHIGH, $01
                !byte CMD_FREQUENCYLOW, $06
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $C5
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $31
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $59
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $06
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $C5
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $31
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $59
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $06
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $C5
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $31
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $59
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $06
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $C5
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $31
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $59
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $06
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $C5
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $31
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $59
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $06
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $C5
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $31
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $59
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $06
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $C5
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $31
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $59
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $06
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $C5
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $31
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $59
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $06
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $C5
                !byte CMD_DELAY, 1
                !byte CMD_FREQUENCYLOW, $31
                !byte CMD_DELAY, 2
                !byte CMD_FREQUENCYLOW, $59
                !byte CMD_DELAY, 2
                !byte CMD_VOICE, SQUARE2_END
                
                !byte CMD_END

;********************************************************************
; Subroutine:   TriggerSoundTune1
;********************************************************************
TriggerSoundTune1
                ldx #VOICE_1
                lda #>soundTune1
                sta soundCmdPtrHigh,X
                lda #<soundTune1
                sta soundCmdPtrLow,X
                lda #TRUE
                sta soundVoiceActive,X
                lda #0
                sta soundCmdBufferIndex,X
                rts

soundTune1
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $A9
                !byte CMD_DELAY, 2
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $01
                !byte CMD_FREQUENCYLOW, $59
                !byte CMD_DELAY, 2
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $01
                !byte CMD_FREQUENCYLOW, $C5
                !byte CMD_DELAY, 2
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $54
                !byte CMD_DELAY, 2
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 4
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $01
                !byte CMD_FREQUENCYLOW, $C5
                !byte CMD_DELAY, 2
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $54
                !byte CMD_DELAY, 5
                !byte CMD_VOICE, SQUARE1_END
                
                !byte CMD_END

;********************************************************************
; Subroutine:   TriggerSoundTune2
;********************************************************************
TriggerSoundTune2
                ldx #VOICE_1
                lda #>soundTune2
                sta soundCmdPtrHigh,X
                lda #<soundTune2
                sta soundCmdPtrLow,X
                lda #TRUE
                sta soundVoiceActive,X
                lda #0
                sta soundCmdBufferIndex,X
                rts

soundTune2
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $E3
                !byte CMD_DELAY, 4
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $C0
                !byte CMD_DELAY, 4
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $AD
                !byte CMD_DELAY, 4
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $01
                !byte CMD_FREQUENCYLOW, $06
                !byte CMD_DELAY, 2
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $A9
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                
                !byte CMD_END

;********************************************************************
; Subroutine:   TriggerSoundScore
;********************************************************************
TriggerSoundScore
                ldx #VOICE_1
                lda #>soundScore
                sta soundCmdPtrHigh,X
                lda #<soundScore
                sta soundCmdPtrLow,X
                lda #TRUE
                sta soundVoiceActive,X
                lda #0
                sta soundCmdBufferIndex,X
                rts

soundScore
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $2A
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $2A
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $2A
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $2A
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $2A
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE1_END
                
                !byte CMD_END

;********************************************************************
; Subroutine:   TriggerSoundBonus
;********************************************************************
TriggerSoundBonus
                ldx #VOICE_2
                lda #>soundBonus
                sta soundCmdPtrHigh,X
                lda #<soundBonus
                sta soundCmdPtrLow,X
                lda #TRUE
                sta soundVoiceActive,X
                lda #0
                sta soundCmdBufferIndex,X
                rts

soundBonus
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE2_START
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $1E
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE2_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE2_START
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $2A
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE2_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE2_START
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $8F
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE2_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE2_START
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $95
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE2_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE2_START
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $A1
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE2_END
                
                !byte CMD_END

;********************************************************************
; Subroutine:   TriggerSoundTune3
;               'Für Elise' (Ludwig van Beethoven)
;********************************************************************
TriggerSoundTune3
                ldx #VOICE_1
                lda #>soundTune3
                sta soundCmdPtrHigh,X
                lda #<soundTune3
                sta soundCmdPtrLow,X
                lda #TRUE
                sta soundVoiceActive,X
                lda #0
                sta soundCmdBufferIndex,X
                rts

soundTune3
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03    ;D#5
                !byte CMD_FREQUENCYLOW, $4C
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03    ;D5
                !byte CMD_FREQUENCYLOW, $42
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03    ;D#5
                !byte CMD_FREQUENCYLOW, $4C
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03    ;D5
                !byte CMD_FREQUENCYLOW, $42
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03    ;D#5
                !byte CMD_FREQUENCYLOW, $4C
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03    ;A#4
                !byte CMD_FREQUENCYLOW, $10
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03    ;C#5
                !byte CMD_FREQUENCYLOW, $36
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03    ;B4
                !byte CMD_FREQUENCYLOW, $1E
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;G#4
                !byte CMD_FREQUENCYLOW, $F2
                !byte CMD_DELAY, 20
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;B3
                !byte CMD_FREQUENCYLOW, $3B
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;D#4
                !byte CMD_FREQUENCYLOW, $98
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;G#4
                !byte CMD_FREQUENCYLOW, $F2
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03    ;A#4
                !byte CMD_FREQUENCYLOW, $10
                !byte CMD_DELAY, 30
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;D#4
                !byte CMD_FREQUENCYLOW, $98
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;G4
                !byte CMD_FREQUENCYLOW, $E3
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03    ;A#4
                !byte CMD_FREQUENCYLOW, $10
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03    ;B4
                !byte CMD_FREQUENCYLOW, $1E
                !byte CMD_DELAY, 20
                !byte CMD_VOICE, SQUARE1_END
                
                !byte CMD_END

;********************************************************************
; Subroutine:   TriggerSoundAlert
;********************************************************************
TriggerSoundAlert
                ldx #VOICE_1
                lda #>soundAlert
                sta soundCmdPtrHigh,X
                lda #<soundAlert
                sta soundCmdPtrLow,X
                lda #TRUE
                sta soundVoiceActive,X
                lda #0
                sta soundCmdBufferIndex,X
                rts

soundAlert
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $76
                !byte CMD_DELAY, 4
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 4
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03
                !byte CMD_FREQUENCYLOW, $1E
                !byte CMD_DELAY, 2
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $3B
                !byte CMD_DELAY, 2
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 6
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $76
                !byte CMD_DELAY, 2
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $00
                !byte CMD_FREQUENCYLOW, $76
                !byte CMD_DELAY, 2
                !byte CMD_VOICE, SQUARE1_END
                
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $54
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $6C
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02
                !byte CMD_FREQUENCYLOW, $83
                !byte CMD_DELAY, 1
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_END

;********************************************************************
; Subroutine:   TriggerSoundTune4
;               'ESTUDIO' from M.A.Robira
;********************************************************************
TriggerSoundTune4
                ldx #VOICE_1
                lda #>soundTune4
                sta soundCmdPtrHigh,X
                lda #<soundTune4
                sta soundCmdPtrLow,X
                lda #TRUE
                sta soundVoiceActive,X
                lda #0
                sta soundCmdBufferIndex,X
                rts

soundTune4
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;F#4
                !byte CMD_FREQUENCYLOW, $D1
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOLUME, $07
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;F#4
                !byte CMD_FREQUENCYLOW, $D1
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOLUME, $06
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;F#4
                !byte CMD_FREQUENCYLOW, $D1
                !byte CMD_DELAY, 11
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOLUME, $06
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;F#4
                !byte CMD_FREQUENCYLOW, $D1
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOLUME, $06
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;E4
                !byte CMD_FREQUENCYLOW, $AD
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOLUME, $06
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;D4
                !byte CMD_FREQUENCYLOW, $83
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOLUME, $06
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;D4
                !byte CMD_FREQUENCYLOW, $83
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOLUME, $06
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;C#4
                !byte CMD_FREQUENCYLOW, $6C
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOLUME, $06
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;B3
                !byte CMD_FREQUENCYLOW, $3B
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOLUME, $06
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;B3
                !byte CMD_FREQUENCYLOW, $3B
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOLUME, $07
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;D4
                !byte CMD_FREQUENCYLOW, $83
                !byte CMD_DELAY, 10
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 2
                
                !byte CMD_VOLUME, $08
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $02    ;F#4
                !byte CMD_FREQUENCYLOW, $D1
                !byte CMD_DELAY, 9
                !byte CMD_VOICE, SQUARE1_END
                !byte CMD_DELAY, 1
                
                !byte CMD_VOLUME, $07
                !byte CMD_VOICE, SQUARE1_START
                !byte CMD_FREQUENCYHIGH, $03    ;B4
                !byte CMD_FREQUENCYLOW, $1E
                !byte CMD_DELAY, 15
                !byte CMD_VOICE, SQUARE1_END
                
                !byte CMD_END

;********************************************************************
; SOUND MODULE END                                                  *
;********************************************************************