;********************************************************************
; TIMER MODULE                                                      *
;********************************************************************
;********************************************************************
; Subroutine:   ResetTimer
; Input:        Y   index for timer (TIMER_0-TIMER_x)
;********************************************************************
ResetTimer
                ;set timer state to inactive
                ;and set counter values to 0
                lda #TIMER_STATE_INACTIVE
                sta timerState,Y
                sta timerLow,Y
                sta timerHigh,Y
                rts

;********************************************************************
; Subroutine:   SetTimerValue
; Input:        Y   index for timer (TIMER_0-TIMER_x)
;               A   timer value high byte
;               X   timer value low byte
;********************************************************************
SetTimerValue
                sta thresholdHigh,Y
                txa
                sta thresholdLow,Y
                rts

;********************************************************************
; Subroutine:   StartTimer
; Input:        Y   index for timer (TIMER_0-TIMER_x)
;********************************************************************
StartTimer
                ;set timer state to active
                lda #TIMER_STATE_ACTIVE
                sta timerState,Y
                rts

;********************************************************************
; Subroutine:   RestartTimer
; Input:        Y   index for timer (TIMER_0-TIMER_x)
;********************************************************************
RestartTimer
                lda #$00
                sta timerLow,Y
                sta timerHigh,Y
                lda #TIMER_STATE_ACTIVE
                sta timerState,Y
                rts

;********************************************************************
; Subroutine:   ResetAllTimers
;********************************************************************
ResetAllTimers
                ldy #TIMER_0
.resetalltimers_loop
                ;reset timer
                lda #TIMER_STATE_INACTIVE
                sta timerState,Y
                sta timerLow,Y
                sta timerHigh,Y
                iny
                cpy #TIMERS_MAX
                bne .resetalltimers_loop
                rts

;********************************************************************
; Subroutine:   UpdateAllTimers
;               Called during raster interrupt
;********************************************************************
UpdateAllTimers
                ldy #TIMER_0
.updatealltimers_loop
                ;increment timer only if state is active
                lda timerState,Y
                cmp #TIMER_STATE_ACTIVE
                bne .updatealltimers_noinc
                clc
                lda #$01
                adc timerLow,Y
                sta timerLow,Y
                lda timerHigh,Y
                adc #$00
                sta timerHigh,Y
.updatealltimers_noinc
                ;check threshold
                lda timerLow,Y
                cmp thresholdLow,Y
                lda timerHigh,Y
                sbc thresholdHigh,Y
                bcc .updatealltimers_notreached
                lda #TIMER_STATE_THRESHOLD
                sta timerState,Y
.updatealltimers_notreached
                iny
                cpy #TIMERS_MAX
                bne .updatealltimers_loop
                rts

;********************************************************************
;software timer data ************************************************
;********************************************************************
timerState
                !fill TIMERS_MAX, TIMER_STATE_INACTIVE

timerHigh
                !fill TIMERS_MAX, 0
timerLow
                !fill TIMERS_MAX, 0

thresholdHigh
                !byte TITLE_TIMER_VALUE_H
                !byte PLAYER_SPEED_TIMER_VALUE_H
                !byte PLAYER_FRAME_TIMER_VALUE_H
                !byte SHIELD_FRAME_TIMER_VALUE_H
                !byte SHIELD_DURATION_TIMER_VALUE_H
                !byte SHIELD_FORBIDDEN_TIMER_VALUE_H
                !byte ALIEN_SPEED_TIMER_VALUE_H
                !byte PLAY_BREAK_TIMER_VALUE_H
                !byte ALIEN_EXPLOSION_FRAME_TIMER_VALUE_H
                !byte GAME_SPEED_TIMER_VALUE_H
                !byte PLAYER_EXPLOSION_FRAME_TIMER_VALUE_H
                !byte ALIEN_FADEIN_FRAME_TIMER_VALUE_H
                !byte ALIEN_ATTACK_INTERVAL_TIMER_VALUE_H
                !byte ALIEN_ATTACK_SPEED_TIMER_VALUE_H
                !byte BOSS_MOVEMENT_SPEED_TIMER_VALUE_H
                !byte ALIENPILOT_FRAME_TIMER_VALUE_H
                !byte BIRDSX_SPEED_TIMER_VALUE_H

thresholdLow
                !byte TITLE_TIMER_VALUE_L
                !byte PLAYER_SPEED_TIMER_VALUE_L
                !byte PLAYER_FRAME_TIMER_VALUE_L
                !byte SHIELD_FRAME_TIMER_VALUE_L
                !byte SHIELD_DURATION_TIMER_VALUE_L
                !byte SHIELD_FORBIDDEN_TIMER_VALUE_L
                !byte ALIEN_SPEED_TIMER_VALUE_L
                !byte PLAY_BREAK_TIMER_VALUE_L
                !byte ALIEN_EXPLOSION_FRAME_TIMER_VALUE_L
                !byte GAME_SPEED_TIMER_VALUE_L
                !byte PLAYER_EXPLOSION_FRAME_TIMER_VALUE_L
                !byte ALIEN_FADEIN_FRAME_TIMER_VALUE_L
                !byte ALIEN_ATTACK_INTERVAL_TIMER_VALUE_L
                !byte ALIEN_ATTACK_SPEED_TIMER_VALUE_L
                !byte BOSS_MOVEMENT_SPEED_TIMER_VALUE_L
                !byte ALIENPILOT_FRAME_TIMER_VALUE_L
                !byte BIRDSX_SPEED_TIMER_VALUE_L

;********************************************************************
; TIMER MODULE END                                                  *
;********************************************************************